package com.example.dams;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.material.textfield.TextInputLayout;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class CadastrarActivity extends AppCompatActivity{

    private static final Pattern VALID_EMAIL_ADDRESS_REGEX = Pattern.compile("^[A-Z0-9._%+-]+@[A-Z0-9.-]+\\.[A-Z]{2,6}$", Pattern.CASE_INSENSITIVE);
    private static final Pattern PASSWORD_PATTERN = Pattern.compile("^(?=.*[0-9])(?=.*[a-z])(?=.*[A-Z])(?=.*[!@#&()–[{}]:;',?/*~$^+=<>]).{6,20}$", Pattern.CASE_INSENSITIVE);
    private FirebaseAuth firebaseAuth;
    private DatabaseReference databaseReference;
    private Button efetuarCadastroButton;
    private TextInputLayout userEditText, emailEditText, senhaEditText, confirmarSenhaEditText;
    private String feedback;
    private String user;
    private String senha;
    private String email;
    private ProgressBar progressBar;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_cadastrar);
        efetuarCadastroButton = findViewById(R.id.cadastrarButton);
        userEditText = findViewById(R.id.cadastrarUsuarioEditText);
        emailEditText = findViewById(R.id.cadastrarEmailEditText);
        senhaEditText = findViewById(R.id.cadastrarSenhaEditText);
        confirmarSenhaEditText = findViewById(R.id.cadastrarConfirmarSenhaEditText);
        progressBar = findViewById(R.id.cadastrarProgressBar);
        databaseReference = FirebaseDatabase.getInstance().getReference().child("users");
        firebaseAuth = FirebaseAuth.getInstance();
        listeners();

    }

    private void listeners() {
        userEditText.getEditText().setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if(!b){
                    validarNomeDeUsuario();
                }
            }
        });
        emailEditText.getEditText().setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if(!b){
                    validarEmail();
                }
            }
        });
        senhaEditText.getEditText().setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if(!b) {
                    validarSenha();
                }
            }
        });
        confirmarSenhaEditText.getEditText().setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if(!b) {
                    validarConfirmarSenha();
                }
            }
        });

        efetuarCadastroButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View view){
                if(validarNomeDeUsuario() && validarEmail() && validarSenha() && validarConfirmarSenha()){
                    progressBar.setVisibility(View.VISIBLE);
                    firebaseAuth.createUserWithEmailAndPassword(email,senha).addOnCompleteListener(new OnCompleteListener<AuthResult>() {
                        @Override
                        public void onComplete(@NonNull Task<AuthResult> task) {
                            if(task.isSuccessful()){
                                databaseReference.child(firebaseAuth.getUid()).addListenerForSingleValueEvent(new ValueEventListener() {
                                    @Override
                                    public void onDataChange(@NonNull DataSnapshot snapshot) {
                                        Usuario usuario = new Usuario(0, user);
                                        usuario.setParametrosPadrao(CadastrarActivity.this);
                                        databaseReference.child(firebaseAuth.getUid()).setValue(usuario).addOnCompleteListener(new OnCompleteListener<Void>() {
                                            @Override
                                            public void onComplete(@NonNull Task<Void> task) {
                                                Toast.makeText(view.getContext(),
                                                        String.format(Locale.getDefault(),view.getResources().getString(R.string.cadastrar_sucesso),user),
                                                        Toast.LENGTH_LONG).show();
                                                Intent it = new Intent(CadastrarActivity.this, LogarActivity.class);
                                                startActivity(it);
                                                progressBar.setVisibility(View.INVISIBLE);
                                            }
                                        }); //New User
                                    }
                                    @Override
                                    public void onCancelled(@NonNull DatabaseError error) {
                                        Toast.makeText(view.getContext(),view.getResources().getString(R.string.erro_bd),Toast.LENGTH_LONG).show();
                                        progressBar.setVisibility(View.INVISIBLE);
                                    }
                                });
                            }else{
                                Toast.makeText(CadastrarActivity.this,
                                        CadastrarActivity.this.getResources().getString(R.string.cadastrar_erro_auth)+task.getException().getMessage(), Toast.LENGTH_LONG).show();
                                progressBar.setVisibility(View.INVISIBLE);
                            }
                        }
                    });
                }
            }
        });
    }

    /*
    private void cursosSpinner(){
        String[] cursos= getResources().getStringArray(R.array.cadastrar_cursos_graduacao);
        List<String> listCursos = new ArrayList<>();
        for(int i = 0; i<cursos.length; i++){
            listCursos.add(cursos[i]);
        }
        ArrayAdapter<String> cursosAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, cursos);
        cursosSpinner.setAdapter(cursosAdapter);
    }*/

    public boolean validarNomeDeUsuario(){
        user = userEditText.getEditText().getText().toString().trim();
        boolean valido = false;
        String noWhiteSpace = "\\A\\w{4,20}\\z";
        if(user.length() < getResources().getInteger(R.integer.num_min_caracteres_username)){
            userEditText.setError(String.format(Locale.getDefault(), getResources().getString(R.string.logar_erro_username_curto), getResources().getInteger(R.integer.num_min_caracteres_username)));
        }else if(user.length() > getResources().getInteger(R.integer.num_max_caracteres_username)){
            userEditText.setError(String.format(Locale.getDefault(), getResources().getString(R.string.logar_erro_username_longo), getResources().getInteger(R.integer.num_max_caracteres_username)));
        }else if(!user.matches(noWhiteSpace)){
            userEditText.setError(getResources().getString(R.string.logar_erro_username_whitespace));
        }else{
            userEditText.setError(null);
            userEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }

    public boolean validarSenha(){
        senha = senhaEditText.getEditText().getText().toString();
        Matcher matcher = PASSWORD_PATTERN.matcher(senha);
        boolean padronizada = matcher.find();
        boolean valido = false;
        if(senha.isEmpty()){
            senhaEditText.setError(getResources().getString(R.string.logar_erro_senha_vazia));
        }else if(!padronizada){
            senhaEditText.setError(getResources().getString(R.string.cadastrar_erro_senha_invalida));
        }else{
            senhaEditText.setError(null);
            senhaEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }

    public boolean validarConfirmarSenha(){
        String confirmarSenha = confirmarSenhaEditText.getEditText().getText().toString();
        boolean valido = false;
        if(confirmarSenha.isEmpty()){
            confirmarSenhaEditText.setError(getResources().getString(R.string.cadastrar_confirmar_senha_hint));
        }else if(!senha.equals(confirmarSenha)){
            confirmarSenhaEditText.setError(getResources().getString(R.string.cadastrar_erro_confirmar_senha));
        }else{
            confirmarSenhaEditText.setError(null);
            confirmarSenhaEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }

    public boolean validarEmail() {
        email = emailEditText.getEditText().getText().toString().trim();
        Matcher matcher = VALID_EMAIL_ADDRESS_REGEX.matcher(email);
        boolean padronizado = matcher.find();
        boolean valido = false;
        if(email.isEmpty()){
            emailEditText.setError(getResources().getString(R.string.cadastrar_erro_email_vazio));
        }else if(!padronizado){
            emailEditText.setError(getResources().getString(R.string.cadastrar_erro_email_invalido));
        }else{
            emailEditText.setError(null);
            emailEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }
}