package com.example.dams;

import android.content.Context;
import android.content.Intent;
import android.content.res.TypedArray;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatDelegate;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.core.content.res.ResourcesCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class CapitulosRVAdapter extends RecyclerView.Adapter<CapitulosRVAdapter.ViewHolder> {

    private Context context;
    private Modulo modulo;
    private List<String> tipoCapitulo;
    private Usuario usuario;
    //private ItemClickListener mClickListener;

    // data is passed into the constructor
    public CapitulosRVAdapter(Context context, Modulo modulo, List<String> tipoCapitulo, Usuario usuario) {
        this.context = context;
        this.modulo = modulo;
        this.tipoCapitulo = tipoCapitulo;
        this.usuario = usuario;

    }

    // inflates the row layout from xml when needed
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.capitulos_rv_row, parent, false);
        return new CapitulosRVAdapter.ViewHolder(view);
    }

    // binds the data to the TextView in each row
    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP){
            holder.capituloLayout.setBackgroundResource(R.drawable.ripple_effect);
        }else{
            AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
            //holder.capituloLayout.setBackgroundResource(R.drawable.capitulos_rv_colors);
        }
        holder.capituloTextView.setText(modulo.getCapitulos().get(position).getTitulo());
        int tipo = Integer.parseInt(tipoCapitulo.get(position));
        if(tipo == 0){
            //Glide.with(context).load(ResourcesCompat.getDrawable(context.getResources(),R.drawable.ic_book,null)).into(holder.tipoImageView);
            //holder.tipoImageView.setImageDrawable(ResourcesCompat.getDrawable(context.getResources(),R.drawable.ic_book,null));
            holder.tipoImageView.setImageResource(R.drawable.ic_book);
        }else if(tipo == 1){
            //Glide.with(context).load(ResourcesCompat.getDrawable(context.getResources(),R.drawable.ic_lightbulb,null)).into(holder.tipoImageView);
            //holder.tipoImageView.setImageDrawable(ResourcesCompat.getDrawable(context.getResources(),R.drawable.ic_lightbulb,null));
            holder.tipoImageView.setImageResource(R.drawable.ic_lightbulb);
        }
        if(usuario.getProgressoCurso().get(modulo.getPosicao()).get(position) == 1){
            //Glide.with(context).load(ResourcesCompat.getDrawable(context.getResources(),R.drawable.ic_check,null)).into(holder.tipoImageView);
            //holder.checkImageView.setImageDrawable(ResourcesCompat.getDrawable(context.getResources(),R.drawable.ic_check,null));
            holder.checkImageView.setImageResource(R.drawable.ic_check);
        }
    }

    // total number of rows
    @Override
    public int getItemCount() {
        return modulo.getCapitulos().size();
    }


    // stores and recycles views as they are scrolled off screen
    public class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
        LinearLayout capituloLayout;
        TextView capituloTextView;
        AppCompatImageView tipoImageView, checkImageView;

        ViewHolder(View itemView) {
            super(itemView);
            capituloLayout = itemView.findViewById(R.id.capituloLayout);
            capituloTextView = itemView.findViewById(R.id.capituloTextView);
            tipoImageView = itemView.findViewById(R.id.capituloTipoImageView);
            checkImageView = itemView.findViewById(R.id.capituloCheckImageView);
            itemView.setOnClickListener(this);
        }

        @Override
        public void onClick(View view) {
            //HtmlCompat conteudo;
            List<String> itens = new ArrayList<>();
            List<String> pergunta1 = new ArrayList<>();
            List<String> pergunta2 = new ArrayList<>();
            List<String> pergunta3 = new ArrayList<>();
            List<String> pergunta4 = new ArrayList<>();
            List<String> pergunta5 = new ArrayList<>();
            int destinoSimulacao = 0;
            Intent it = new Intent();
            Bundle params = new Bundle();
            switch (modulo.getPosicao()) {
                case 0: {
                    switch (getBindingAdapterPosition()) {
                        case 0: {
                            //conteudo = HtmlCompat.fromHtml(context.getResources().getString(R.string.conceitos_barragens));
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_pergunta5)));
                            break;
                        }
                        case 1: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_gravidade_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_gravidade_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_gravidade_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_gravidade_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_gravidade_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_barragens_gravidade_pergunta5)));
                            break;
                        }
                        case 2: {
                            destinoSimulacao = 0; //Simular Centro de Massa
                            break;
                        }
                    }
                    break;
                }
                case 1: {
                    switch (getBindingAdapterPosition()) {
                        case 0: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_hidro_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_hidro_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_hidro_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_hidro_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_hidro_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_hidro_pergunta5)));
                            break;
                        }
                        case 1: {
                            destinoSimulacao = 1; //Simular Pressões
                            break;
                        }
                    }
                    break;
                }
                case 2: {
                    switch (getBindingAdapterPosition()) {
                        case 0: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_cc_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_cc_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_cc_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_cc_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_cc_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_cc_pergunta5)));
                            break;
                        }
                        case 1: {
                            destinoSimulacao = 2; //Simular CC
                            break;
                        }
                    }
                    break;
                }
                case 3: {
                    switch (getBindingAdapterPosition()) {
                        case 0: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_forcas_gravitacionais_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_forcas_gravitacionais_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_forcas_gravitacionais_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_forcas_gravitacionais_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_forcas_gravitacionais_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_forcas_gravitacionais_pergunta5)));
                            break;
                        }
                        case 1: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_subpressao_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_subpressao_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_subpressao_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_subpressao_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_subpressao_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_subpressao_pergunta5)));
                            break;
                        }
                        case 2: {
                            destinoSimulacao = 3; //Simular Flutuação
                            break;
                        }
                    }
                    break;
                }
                case 4: {
                    switch (getBindingAdapterPosition()) {
                        case 0: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_estabilizantes_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_estabilizantes_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_estabilizantes_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_estabilizantes_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_estabilizantes_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_estabilizantes_pergunta5)));
                            break;
                        }
                        case 1: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_desestabilizantes_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_desestabilizantes_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_desestabilizantes_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_desestabilizantes_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_desestabilizantes_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_desestabilizantes_pergunta5)));
                            break;
                        }
                        case 2: {
                            destinoSimulacao = 4; //Simular Tombamento
                            break;
                        }
                    }
                    break;
                }
                case 5: {
                    switch (getBindingAdapterPosition()) {
                        case 0: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_atrito_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_atrito_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_atrito_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_atrito_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_atrito_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_atrito_pergunta5)));
                            break;
                        }
                        case 1: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_coesao_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_coesao_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_coesao_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_coesao_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_coesao_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_coesao_pergunta5)));
                            break;
                        }
                        case 2: {
                            destinoSimulacao = 5; //Simular Deslizamento
                            break;
                        }
                    }
                    break;
                }
                case 6: {
                    switch (getBindingAdapterPosition()) {
                        case 0: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_tensoes_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_tensoes_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_tensoes_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_tensoes_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_tensoes_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_tensoes_pergunta5)));
                            break;
                        }
                        case 1: {
                            itens.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_fissura_itens)));
                            pergunta1.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_fissura_pergunta1)));
                            pergunta2.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_fissura_pergunta2)));
                            pergunta3.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_fissura_pergunta3)));
                            pergunta4.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_fissura_pergunta4)));
                            pergunta5.addAll(Arrays.asList(context.getResources().getStringArray(R.array.conceitos_fissura_pergunta5)));
                            break;
                        }
                        case 2: {
                            destinoSimulacao = 6; //Simular Tensões na Base
                            break;
                        }
                    }
                    break;
                }
            }
            if(tipoCapitulo.get(getBindingAdapterPosition()).equals("0")){ //Conceitual
                Conceito conceito = new Conceito(modulo.getPosicao(), getBindingAdapterPosition(), modulo.getCapitulos().get(getBindingAdapterPosition()).getTitulo(), itens.get(0), itens.get(1),
                        pergunta1, pergunta2, pergunta3, pergunta4, pergunta5);
                it.setClass(context, ConceitosActivity.class);
                params.putParcelable("conceito", conceito);
                it.putExtras(params);
                context.startActivity(it);
            }
            if(tipoCapitulo.get(getBindingAdapterPosition()).equals("1")){ //Simulação
                if(destinoSimulacao == 2){
                    it.setClass(context, SimularCCActivity.class); //SimularCCActivity
                }else{
                    it.setClass(context, SelecionarSecaoActivity.class); //Demais Simulações
                    params.putStringArrayList("unidades", (ArrayList<String>)usuario.getUnidades());
                    params.putStringArrayList("cmin_ccn", (ArrayList<String>)usuario.getCmin().get(0));
                    params.putStringArrayList("cmin_cce", (ArrayList<String>)usuario.getCmin().get(1));
                    params.putStringArrayList("cmin_ccl", (ArrayList<String>)usuario.getCmin().get(2));
                    params.putStringArrayList("cmin_ccc", (ArrayList<String>)usuario.getCmin().get(3));
                }
                params.putString("titulo", modulo.getCapitulos().get(getBindingAdapterPosition()).getTitulo());
                params.putInt("posModulo", modulo.getPosicao());
                params.putInt("posCapitulo", getBindingAdapterPosition());
                params.putInt("destino", destinoSimulacao);
                it.putExtras(params);
                context.startActivity(it);
            }
            //Toast.makeText(context, "You clicked " + modulo.getCapitulos().get(getBindingAdapterPosition()).getTitulo() + " on row number " + getBindingAdapterPosition(), Toast.LENGTH_SHORT).show();
            //if (mClickListener != null) mClickListener.onItemClick(view, getAdapterPosition());
        }
    }
}
    /*// convenience method for getting data at click position
    String getItem(int id) {
        return capitulos.get(id);
    }

    // allows clicks events to be caught
    void setClickListener(ItemClickListener itemClickListener) {
        this.mClickListener = itemClickListener;
    }

    // parent activity will implement this method to respond to click events
    public class ItemClickListener {
        void onItemClick(View view, int position){
            Toast.makeText(context, "You clicked " + capitulos.get(position) + " on row number " + position, Toast.LENGTH_SHORT).show();
        }
    }*/
    /*TypedArray ta= context.getResources().obtainTypedArray(R.array.conceitos_barragens);
                            CharSequence[] itens = ta.getTextArray(0);
                            CharSequence[] perguntaCharSequence;
                            List<String> pergunta = new ArrayList<>();
                            List<List<String>> perguntas = new ArrayList<>();
                            for(int i = 1; i<ta.length(); i++){
                                perguntaCharSequence = ta.getTextArray(i);
                                for(int j = 0; j < perguntaCharSequence.length; j++){
                                    pergunta.add(perguntaCharSequence[j].toString());
                                }
                                perguntas.add(pergunta);
                                pergunta.clear();
                            }
                            ta.recycle();*/