package com.example.dams;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import java.math.RoundingMode;
import java.text.NumberFormat;
import java.util.Locale;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link CentroDeMassaFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class CentroDeMassaFragment extends Fragment {

    private EditText pesoEditText, cgxEditText, cgyEditText;
    private Button simularButton;
    private String feedback;
    private SimulacaoViewModel simulacaoViewModel;
    private Number pesoU, cgxU, cgyU;
    private NumberFormat format;

    public CentroDeMassaFragment() {
        // Required empty public constructor
    }

    // TODO: Rename and change types and number of parameters
    public static CentroDeMassaFragment newInstance() {
        return new CentroDeMassaFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        simulacaoViewModel = new ViewModelProvider(getActivity()).get(SimulacaoViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_centro_de_massa, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        pesoEditText = getView().findViewById(R.id.centroDeMassaPesoEditText);
        cgxEditText = getView().findViewById(R.id.centroDeMassaCGXEditText);
        cgyEditText = getView().findViewById(R.id.centroDeMassaCGYEditText);
        simularButton = getView().findViewById(R.id.centroDeMassaButton);
        format = NumberFormat.getInstance(Locale.getDefault());
        simulacaoViewModel.getSimulacao().getValue().simular();
        listeners();
    }


    private void listeners() {
        pesoEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_aviso_unidade),
                            v.getContext().getResources().getString(R.string.simular_centro_massa_peso_hint), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(2));
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

        cgxEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_aviso_unidade),
                            v.getContext().getResources().getString(R.string.simular_cgx_hint), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(0));
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

        cgyEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_aviso_unidade),
                            v.getContext().getResources().getString(R.string.simular_cgy_hint), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(0));
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

        simularButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try{
                    pesoU = format.parse(pesoEditText.getText().toString());
                    cgxU = format.parse(cgxEditText.getText().toString());
                    cgyU = format.parse(cgyEditText.getText().toString());
                    if(pesoU.doubleValue() <= 0){
                        feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_feedback_parametro_zerado),
                                v.getContext().getResources().getString(R.string.simular_centro_massa_peso_hint));
                    }else if(cgxU.doubleValue() <= 0){
                        feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_feedback_parametro_zerado),
                                v.getContext().getResources().getString(R.string.simular_cgx_hint));
                    }else if(cgyU.doubleValue() <= 0){
                        feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_feedback_parametro_zerado),
                                v.getContext().getResources().getString(R.string.simular_cgy_hint));
                    }else{
                        simulacaoViewModel.getSimulacao().getValue().obterFeedbackCentroMassa(pesoU.doubleValue(), cgxU.doubleValue(), cgyU.doubleValue());
                        simulacaoViewModel.setSimulacao(simulacaoViewModel.getSimulacao().getValue());
                        feedback="";
                    }
                }catch (Exception e){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.aviso_campos_nao_preenchidos));
                }

                if(!feedback.isEmpty()){
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });
    }
}