package com.example.dams;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Adapter;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.ListAdapter;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.res.ResourcesCompat;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import com.bumptech.glide.Glide;
import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInClient;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.material.switchmaterial.SwitchMaterial;
import com.google.android.material.textfield.TextInputLayout;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link DadosFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class DadosFragment extends Fragment{

    //Fragment Dados
    private static final Pattern VALID_EMAIL_ADDRESS_REGEX = Pattern.compile("^[A-Z0-9._%+-]+@[A-Z0-9.-]+\\.[A-Z]{2,6}$", Pattern.CASE_INSENSITIVE);
    private static final Pattern PASSWORD_PATTERN = Pattern.compile("^(?=.*[0-9])(?=.*[a-z])(?=.*[A-Z])(?=.*[!@#&()–[{}]:;',?/*~$^+=<>]).{6,20}$", Pattern.CASE_INSENSITIVE);
    private FirebaseAuth firebaseAuth;
    private FirebaseUser firebaseUser;
    private DatabaseReference databaseReference;
    private GoogleSignInClient googleSignInClient;
    private ImageView userImageView;
    private Button atualizarButton, modificarSenhaButton, logoutButton, saveButton;
    private SwitchMaterial cminSwitch;
    private AutoCompleteTextView ccSpinner;
    private TextInputLayout csfEditText, cstEditText, csdaEditText, csdcEditText, porcCompEditText, userEditText, emailEditText, senhaAtualEditText, novaSenhaEditText;
    private NumberFormat format;
    private ProgressBar progressBar;
    private UsuarioViewModel usuarioViewModel;
    private String newUsername, novoEmail, novaSenha, senhaAtual;
    private Number csf, cst, csda, csdc, porcComp;
    private List<List<String>> standardCMin, cMinMax;
    private List<String> casos;
    private int ccSelected;

    public DadosFragment() {
        // Required empty public constructor
    }

    public static DadosFragment newInstance() {
        return new DadosFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        firebaseAuth = FirebaseAuth.getInstance();
        firebaseUser = firebaseAuth.getCurrentUser();
        databaseReference = FirebaseDatabase.getInstance().getReference().child("users").child(firebaseAuth.getUid());
        usuarioViewModel = new ViewModelProvider(getActivity()).get(UsuarioViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_dados, container, false);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        //Fragment Dados
        progressBar = getView().findViewById(R.id.dadosProgressBar);
        userImageView = getView().findViewById(R.id.dadosImageView);
        ccSpinner = getView().findViewById(R.id.dadosCCSpinner);
        cminSwitch = getView().findViewById(R.id.dadosCminSwitch);
        csfEditText = getView().findViewById(R.id.dadosCSFEditText);
        cstEditText = getView().findViewById(R.id.dadosCSTEditText);
        csdaEditText = getView().findViewById(R.id.dadosCSDaEditText);
        csdcEditText = getView().findViewById(R.id.dadosCSDcEditText);
        porcCompEditText = getView().findViewById(R.id.dadosPorcCompEditText);
        userEditText = getView().findViewById(R.id.dadosUsuarioEditText);
        emailEditText = getView().findViewById(R.id.dadosEmailEditText);
        senhaAtualEditText = getView().findViewById(R.id.dadosSenhaAtualEditText);
        novaSenhaEditText = getView().findViewById(R.id.dadosNovaSenhaEditText);
        atualizarButton = getView().findViewById(R.id.dadosAtualizarButton);
        modificarSenhaButton = getView().findViewById(R.id.dadosModificarSenhaButton);
        logoutButton = getView().findViewById(R.id.dadosLogoutButton);
        saveButton  = getView().findViewById(R.id.dadosSalvarConfigButton);

        format = NumberFormat.getInstance(Locale.getDefault());
        casos = Arrays.asList(this.getResources().getStringArray(R.array.casos_eletrobras));
        ArrayAdapter<String> ccAdapter = new ArrayAdapter<String>(getView().getContext(), android.R.layout.simple_list_item_1, casos);
        ccSpinner.setAdapter(ccAdapter);
        if(firebaseUser.getPhotoUrl() != null){
            Glide.with(view.getContext()).load(firebaseUser.getPhotoUrl()).into(userImageView);
        }else{
            Glide.with(view.getContext()).load(ResourcesCompat.getDrawable(this.getResources(),R.drawable.dams_logo,null)).into(userImageView);
        }
        if(usuarioViewModel.getUsuario().getValue() != null){
            standardCMin = usuarioViewModel.getUsuario().getValue().obterCMINPadrao(getView().getContext());
            cMinMax = usuarioViewModel.getUsuario().getValue().obterCMINMax(getView().getContext());
            ccSelected = 0;
            ccSpinner.setText(ccAdapter.getItem(ccSelected), false);
            onItemSelectedInSpinner(ccSelected);
            userEditText.getEditText().setText(usuarioViewModel.getUsuario().getValue().getUsername());
            emailEditText.getEditText().setText(firebaseUser.getEmail());
            switch (usuarioViewModel.getUsuario().getValue().getLoginMethod()){
                case 1:{
                    googleSignInClient = GoogleSignIn.getClient(getView().getContext(), GoogleSignInOptions.DEFAULT_SIGN_IN);
                    emailEditText.setEnabled(false);
                    senhaAtualEditText.setEnabled(false);
                    novaSenhaEditText.setEnabled(false);
                    break;
                }
            }
        }

        listeners();
    }

    private void onItemSelectedInSpinner(int cc){
        if(standardCMin.get(cc).equals(usuarioViewModel.getUsuario().getValue().getCmin().get(cc))){
            cminSwitch.setChecked(true);
            setStandardMinimumParameters(cc);
            setParametersEnabled(false);
        }else{
            cminSwitch.setChecked(false);
            setUserMinimumParameters(cc);
            setParametersEnabled(true);
        }
    }
    private void setUserMinimumParameters(int cc){
        csfEditText.getEditText().setText(usuarioViewModel.getUsuario().getValue().getCmin().get(cc).get(0));
        cstEditText.getEditText().setText(usuarioViewModel.getUsuario().getValue().getCmin().get(cc).get(1));
        csdaEditText.getEditText().setText(usuarioViewModel.getUsuario().getValue().getCmin().get(cc).get(2));
        csdcEditText.getEditText().setText(usuarioViewModel.getUsuario().getValue().getCmin().get(cc).get(3));
        porcCompEditText.getEditText().setText(usuarioViewModel.getUsuario().getValue().getCmin().get(cc).get(4));
    }

    private void setStandardMinimumParameters(int cc){
        csfEditText.getEditText().setText(standardCMin.get(cc).get(0));
        cstEditText.getEditText().setText(standardCMin.get(cc).get(1));
        csdaEditText.getEditText().setText(standardCMin.get(cc).get(2));
        csdcEditText.getEditText().setText(standardCMin.get(cc).get(3));
        porcCompEditText.getEditText().setText(standardCMin.get(cc).get(4));
    }

    private void setParametersEnabled(boolean active){
        csfEditText.setEnabled(active);
        cstEditText.setEnabled(active);
        csdaEditText.setEnabled(active);
        csdcEditText.setEnabled(active);
        porcCompEditText.setEnabled(active);
    }

    private void listeners() {

        ccSpinner.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                ccSelected = i;
                onItemSelectedInSpinner(ccSelected);
                clearParametersErrors();
                saveButton.setEnabled(false);
            }
        });

        cminSwitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if(b){
                    setStandardMinimumParameters(ccSelected);
                    setParametersEnabled(false);
                }else{
                    setUserMinimumParameters(ccSelected);
                    setParametersEnabled(true);
                }
            }
        });
        View.OnFocusChangeListener salvarParametrosFocusChanged = new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if(!b){
                    if(view.getId() == R.id.dadosCSFEditTextIn){
                        validarCSF(ccSelected);
                    }else if(view.getId() == R.id.dadosCSTEditTextIn){
                        validarCST(ccSelected);
                    }else if(view.getId() == R.id.dadosCSDaEditTextIn){
                        validarCSDa(ccSelected);
                    }else if(view.getId() == R.id.dadosCSDcEditTextIn){
                        validarCSDc(ccSelected);
                    }else if(view.getId() == R.id.dadosPorcCompEditTextIn){
                        validarPorcComp(ccSelected);
                    }
                }
            }
        };
        csfEditText.getEditText().setOnFocusChangeListener(salvarParametrosFocusChanged);
        cstEditText.getEditText().setOnFocusChangeListener(salvarParametrosFocusChanged);
        csdaEditText.getEditText().setOnFocusChangeListener(salvarParametrosFocusChanged);
        csdcEditText.getEditText().setOnFocusChangeListener(salvarParametrosFocusChanged);
        porcCompEditText.getEditText().setOnFocusChangeListener(salvarParametrosFocusChanged);

        View.OnFocusChangeListener atualizarDadosFocusChanged = new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if(!b){
                    if(view.getId() == R.id.dadosUsuarioEditTextIn){
                        validarNomeDeUsuario();
                    }else if(view.getId() == R.id.dadosEmailEditTextIn){
                        validarEmail();
                    }
                }
            }
        };
        userEditText.getEditText().setOnFocusChangeListener(atualizarDadosFocusChanged);
        emailEditText.getEditText().setOnFocusChangeListener(atualizarDadosFocusChanged);

        View.OnFocusChangeListener alterarSenhaFocusChanged = new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if(view.getId() == R.id.dadosSenhaAtualEditTextIn){
                    validarSenhaAtual();
                }else if(view.getId() == R.id.dadosNovaSenhaEditTextIn){
                    validarNovaSenha();
                }
            }
        };
        senhaAtualEditText.getEditText().setOnFocusChangeListener(alterarSenhaFocusChanged);
        novaSenhaEditText.getEditText().setOnFocusChangeListener(alterarSenhaFocusChanged);

        TextWatcher salvarParametrosTextWatcher = new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }
            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                try{
                    if(format.parse(csfEditText.getEditText().getText().toString()).doubleValue() == format.parse(usuarioViewModel.getUsuario().getValue().getCmin().get(ccSelected).get(0)).doubleValue() &&
                            format.parse(cstEditText.getEditText().getText().toString()).doubleValue() == format.parse(usuarioViewModel.getUsuario().getValue().getCmin().get(ccSelected).get(1)).doubleValue() &&
                            format.parse(csdaEditText.getEditText().getText().toString()).doubleValue() == format.parse(usuarioViewModel.getUsuario().getValue().getCmin().get(ccSelected).get(2)).doubleValue() &&
                            format.parse(csdcEditText.getEditText().getText().toString()).doubleValue() == format.parse(usuarioViewModel.getUsuario().getValue().getCmin().get(ccSelected).get(3)).doubleValue() &&
                            format.parse(porcCompEditText.getEditText().getText().toString()).doubleValue() == format.parse(usuarioViewModel.getUsuario().getValue().getCmin().get(ccSelected).get(4)).doubleValue()){
                        saveButton.setEnabled(false);
                    }else{
                        saveButton.setEnabled(true);
                    }
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable editable) {
            }
        };
        csfEditText.getEditText().addTextChangedListener(salvarParametrosTextWatcher);
        cstEditText.getEditText().addTextChangedListener(salvarParametrosTextWatcher);
        csdaEditText.getEditText().addTextChangedListener(salvarParametrosTextWatcher);
        csdcEditText.getEditText().addTextChangedListener(salvarParametrosTextWatcher);
        porcCompEditText.getEditText().addTextChangedListener(salvarParametrosTextWatcher);

        TextWatcher atualizarDadosTextWatcher = new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }
            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                try{
                    if(userEditText.getEditText().getText().toString().trim().equals(usuarioViewModel.getUsuario().getValue().getUsername()) &&
                            emailEditText.getEditText().getText().toString().trim().equals(firebaseUser.getEmail())){
                        atualizarButton.setEnabled(false);
                    }else{
                        atualizarButton.setEnabled(true);
                    }
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable editable) {
            }
        };
        userEditText.getEditText().addTextChangedListener(atualizarDadosTextWatcher);
        emailEditText.getEditText().addTextChangedListener(atualizarDadosTextWatcher);

        TextWatcher alterarSenhaTextWatcher = new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }
            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                try{
                    if(!senhaAtualEditText.getEditText().getText().toString().isEmpty() &&
                            !novaSenhaEditText.getEditText().getText().toString().isEmpty()){
                        modificarSenhaButton.setEnabled(true);
                    }else{
                        modificarSenhaButton.setEnabled(false);
                    }
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable editable) {
            }
        };
        senhaAtualEditText.getEditText().addTextChangedListener(alterarSenhaTextWatcher);
        novaSenhaEditText.getEditText().addTextChangedListener(alterarSenhaTextWatcher);

        logoutButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                progressBar.setVisibility(View.VISIBLE);
                switch (usuarioViewModel.getUsuario().getValue().getLoginMethod()){
                    case 0:{
                        firebaseAuth.signOut();
                        Toast.makeText(view.getContext(), view.getResources().getString(R.string.atualizarDados_logout),Toast.LENGTH_LONG).show();
                        Intent it = new Intent(view.getContext().getApplicationContext(), IniciarActivity.class);
                        it.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                        startActivity(it);
                        break;
                    }
                    case 1:{
                        googleSignInClient.signOut().addOnCompleteListener(new OnCompleteListener<Void>() {
                            @Override
                            public void onComplete(@NonNull Task<Void> task) {
                                if(task.isSuccessful()){
                                    firebaseAuth.signOut();
                                    Toast.makeText(view.getContext(), view.getResources().getString(R.string.atualizarDados_logout_google),Toast.LENGTH_LONG).show();
                                    Intent it = new Intent(view.getContext().getApplicationContext(), IniciarActivity.class);
                                    it.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                                    startActivity(it);
                                }
                            }
                        });
                        break;
                    }
                }
            }
        });

        saveButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if(validarCSF(ccSelected) && validarCST(ccSelected) && validarCSDa(ccSelected) &&
                        validarCSDc(ccSelected) && validarPorcComp(ccSelected)){
                    progressBar.setVisibility(View.VISIBLE);
                    List<String> newCmin = new ArrayList<>();
                    newCmin.add(String.format(Locale.getDefault(),"%.2f",csf.doubleValue()));
                    newCmin.add(String.format(Locale.getDefault(),"%.2f",cst.doubleValue()));
                    newCmin.add(String.format(Locale.getDefault(),"%.2f",csda.doubleValue()));
                    newCmin.add(String.format(Locale.getDefault(),"%.2f",csdc.doubleValue()));
                    newCmin.add(String.format(Locale.getDefault(),"%.2f",porcComp.doubleValue()));
                    databaseReference.child("cmin").child(Integer.toString(ccSelected)).setValue(newCmin).addOnCompleteListener(new OnCompleteListener<Void>() {
                        @Override
                        public void onComplete(@NonNull Task<Void> task) {
                            progressBar.setVisibility(View.INVISIBLE);
                            if(task.isSuccessful()){
                                usuarioViewModel.getUsuario().getValue().getCmin().set(ccSelected, newCmin);
                                usuarioViewModel.setUsuario(usuarioViewModel.getUsuario().getValue());
                                csfEditText.getEditText().setText(newCmin.get(0));
                                cstEditText.getEditText().setText(newCmin.get(1));
                                csdaEditText.getEditText().setText(newCmin.get(2));
                                csdcEditText.getEditText().setText(newCmin.get(3));
                                porcCompEditText.getEditText().setText(newCmin.get(4));
                                Toast.makeText(view.getContext(), String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametros_atualizados),
                                        getResources().getString(R.string.simular_cc_hint),
                                        casos.get(ccSelected)), Toast.LENGTH_LONG).show();
                            }else{
                                Toast.makeText(view.getContext(), task.getException().getMessage(), Toast.LENGTH_LONG).show();
                            }
                        }
                    });
                }
            }
        });
        atualizarButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if(validarNomeDeUsuario() && validarEmail()){
                    progressBar.setVisibility(View.VISIBLE);
                    if(!novoEmail.equals(firebaseUser.getEmail())){
                        firebaseUser.updateEmail(novoEmail).addOnCompleteListener(new OnCompleteListener<Void>() {
                            @Override
                            public void onComplete(@NonNull Task<Void> task) {
                                progressBar.setVisibility(View.INVISIBLE);
                                if(task.isSuccessful()){
                                    emailEditText.getEditText().setText(firebaseUser.getEmail());
                                    atualizarButton.setEnabled(false);
                                    Toast.makeText(view.getContext(), view.getContext().getResources().getString(R.string.atualizarDados_novo_email_sucesso), Toast.LENGTH_LONG).show();
                                }else{
                                    Toast.makeText(view.getContext(), task.getException().getMessage(), Toast.LENGTH_LONG).show();
                                }
                            }
                        });
                    }
                    if(!newUsername.equals(usuarioViewModel.getUsuario().getValue().getUsername())){
                        databaseReference.child("username").setValue(newUsername).addOnCompleteListener(new OnCompleteListener<Void>() {
                            @Override
                            public void onComplete(@NonNull Task<Void> task) {
                                progressBar.setVisibility(View.INVISIBLE);
                                if(task.isSuccessful()){
                                    usuarioViewModel.getUsuario().getValue().setUsername(newUsername);
                                    usuarioViewModel.setUsuario(usuarioViewModel.getUsuario().getValue());
                                    userEditText.getEditText().setText(usuarioViewModel.getUsuario().getValue().getUsername());
                                    atualizarButton.setEnabled(false);
                                    Toast.makeText(view.getContext(), view.getContext().getResources().getString(R.string.atualizarDados_novo_username_sucesso), Toast.LENGTH_LONG).show();
                                }else{
                                    Toast.makeText(view.getContext(), task.getException().getMessage(), Toast.LENGTH_LONG).show();
                                }
                            }
                        });
                    }
                }
            }
        });

        modificarSenhaButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if(validarSenhaAtual() && validarNovaSenha()){
                    progressBar.setVisibility(View.VISIBLE);
                    firebaseUser.updatePassword(novaSenha).addOnCompleteListener(new OnCompleteListener<Void>() {
                        @Override
                        public void onComplete(@NonNull Task<Void> task) {
                            progressBar.setVisibility(View.INVISIBLE);
                            if(task.isSuccessful()){
                                senhaAtualEditText.getEditText().setText("");
                                novaSenhaEditText.getEditText().setText("");
                                modificarSenhaButton.setEnabled(false);
                                Toast.makeText(view.getContext(), view.getContext().getResources().getString(R.string.atualizarDados_nova_senha_sucesso), Toast.LENGTH_LONG).show();
                            }else{
                                Toast.makeText(view.getContext(), task.getException().getMessage(), Toast.LENGTH_LONG).show();
                            }
                        }
                    });
                }
            }
        });
    }
    /*cst = format.parse(cstEditText.getEditText().getText().toString());
    csda = format.parse(csdaEditText.getEditText().getText().toString());
    csdc = format.parse(csdcEditText.getEditText().getText().toString());
    porcComp = format.parse(porcCompEditText.getEditText().getText().toString());*/
    public boolean validarCSF(int cc){
        boolean valido = false;
        try{
            csf = format.parse(csfEditText.getEditText().getText().toString());
            if(csf.doubleValue() < 1){
                csfEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_abaixo_do_minimo),
                        getResources().getString(R.string.simular_flutuacao_csf_hint), 1));
            }else if(csf.doubleValue() > format.parse(cMinMax.get(cc).get(0)).doubleValue()){
                csfEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_acima_do_maximo),
                        getResources().getString(R.string.simular_flutuacao_csf_hint),
                        cMinMax.get(cc).get(0),
                        getResources().getString(R.string.simular_cc_hint),
                        casos.get(cc)));
            }else{
                csfEditText.setError(null);
                csfEditText.setErrorEnabled(false);
                valido = true;
            }
        }catch (Exception e){
            csfEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_invalido),
                    getResources().getString(R.string.simular_flutuacao_csf_hint)));
        }
        return valido;
    }

    public boolean validarCST(int cc){
        boolean valido = false;
        try{
            cst = format.parse(cstEditText.getEditText().getText().toString());
            if(cst.doubleValue() < 1){
                cstEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_abaixo_do_minimo),
                        getResources().getString(R.string.simular_tombamento_cst_hint), 1));
            }else if(cst.doubleValue() > format.parse(cMinMax.get(cc).get(1)).doubleValue()){
                cstEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_acima_do_maximo),
                        getResources().getString(R.string.simular_tombamento_cst_hint),
                        cMinMax.get(cc).get(1),
                        getResources().getString(R.string.simular_cc_hint),
                        casos.get(cc)));
            }else{
                cstEditText.setError(null);
                cstEditText.setErrorEnabled(false);
                valido = true;
            }
        }catch (Exception e){
            cstEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_invalido),
                    getResources().getString(R.string.simular_tombamento_cst_hint)));
        }
        return valido;
    }

    public boolean validarCSDa(int cc){
        boolean valido = false;
        try{
            csda = format.parse(csdaEditText.getEditText().getText().toString());
            if(csda.doubleValue() < 1){
                csdaEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_abaixo_do_minimo),
                        getResources().getString(R.string.simular_deslizamento_atrito_csd_hint), 1));
            }else if(csda.doubleValue() > format.parse(cMinMax.get(cc).get(2)).doubleValue()){
                csdaEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_acima_do_maximo),
                        getResources().getString(R.string.simular_deslizamento_atrito_csd_hint),
                        cMinMax.get(cc).get(2),
                        getResources().getString(R.string.simular_cc_hint),
                        casos.get(cc)));
            }else{
                csdaEditText.setError(null);
                csdaEditText.setErrorEnabled(false);
                valido = true;
            }
        }catch (Exception e){
            csdaEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_invalido),
                    getResources().getString(R.string.simular_deslizamento_atrito_csd_hint)));
        }
        return valido;
    }

    public boolean validarCSDc(int cc){
        boolean valido = false;
        try{
            csdc = format.parse(csdcEditText.getEditText().getText().toString());
            if(csdc.doubleValue() < 1){
                csdcEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_abaixo_do_minimo),
                        getResources().getString(R.string.simular_deslizamento_coesao_csd_hint), 1));
            }else if(csdc.doubleValue() > format.parse(cMinMax.get(cc).get(3)).doubleValue()){
                csdcEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_acima_do_maximo),
                        getResources().getString(R.string.simular_deslizamento_coesao_csd_hint),
                        cMinMax.get(cc).get(3),
                        getResources().getString(R.string.simular_cc_hint),
                        casos.get(cc)));
            }else{
                csdcEditText.setError(null);
                csdcEditText.setErrorEnabled(false);
                valido = true;
            }
        }catch (Exception e){
            csdcEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_invalido),
                    getResources().getString(R.string.simular_deslizamento_coesao_csd_hint)));
        }
        return valido;
    }

    public boolean validarPorcComp(int cc){
        boolean valido = false;
        try{
            porcComp = format.parse(porcCompEditText.getEditText().getText().toString());
            if(porcComp.doubleValue() < 0){
                porcCompEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_abaixo_do_minimo),
                        getResources().getString(R.string.simular_tensoes_base_comp_hint), 0));
            }else if(porcComp.doubleValue() > format.parse(cMinMax.get(cc).get(4)).doubleValue()){
                porcCompEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_parametro_acima_do_maximo),
                        getResources().getString(R.string.simular_tensoes_base_comp_hint),
                        cMinMax.get(cc).get(4),
                        getResources().getString(R.string.simular_cc_hint),
                        casos.get(cc)));
            }else{
                porcCompEditText.setError(null);
                porcCompEditText.setErrorEnabled(false);
                valido = true;
            }
        }catch (Exception e){
            porcCompEditText.setError(String.format(Locale.getDefault(), getContext().getResources().getString(R.string.atualizarDados_invalido),
                    getResources().getString(R.string.simular_tensoes_base_comp_hint)));
        }
        return valido;
    }

    public boolean validarEmail() {
        novoEmail = emailEditText.getEditText().getText().toString().trim();
        Matcher matcher = VALID_EMAIL_ADDRESS_REGEX.matcher(novoEmail);
        boolean padronizado = matcher.find();
        boolean valido = false;
        if(novoEmail.isEmpty()){
            emailEditText.setError(getResources().getString(R.string.cadastrar_erro_email_vazio));
        }else if(!padronizado){
            emailEditText.setError(getResources().getString(R.string.cadastrar_erro_email_invalido));
        }else{
            emailEditText.setError(null);
            emailEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }

    public boolean validarSenhaAtual(){
        senhaAtual = senhaAtualEditText.getEditText().getText().toString();
        boolean valido = false;
        if(senhaAtual.isEmpty()){
            senhaAtualEditText.setError(getResources().getString(R.string.atualizarDados_senha_atual_vazia));
        }else{
            senhaAtualEditText.setError(null);
            senhaAtualEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }
    public boolean validarNovaSenha(){
        novaSenha = novaSenhaEditText.getEditText().getText().toString();
        Matcher matcher = PASSWORD_PATTERN.matcher(novaSenha);
        boolean padronizada = matcher.find();
        boolean valido = false;
        if(novaSenha.isEmpty()){
            novaSenhaEditText.setError(getResources().getString(R.string.atualizarDados_nova_senha_vazia));
        }else if(!padronizada){
            novaSenhaEditText.setError(getResources().getString(R.string.cadastrar_erro_senha_invalida));
        }else{
            novaSenhaEditText.setError(null);
            novaSenhaEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }

    public boolean validarNomeDeUsuario(){
        newUsername = userEditText.getEditText().getText().toString().trim();
        boolean valido = false;
        String noWhiteSpace = "\\A\\w{4,20}\\z";
        if(newUsername.length() < getResources().getInteger(R.integer.num_min_caracteres_username)){
            userEditText.setError(String.format(Locale.getDefault(), getResources().getString(R.string.logar_erro_username_curto), getResources().getInteger(R.integer.num_min_caracteres_username)));
        }else if(newUsername.length() > getResources().getInteger(R.integer.num_max_caracteres_username)){
            userEditText.setError(String.format(Locale.getDefault(), getResources().getString(R.string.logar_erro_username_longo), getResources().getInteger(R.integer.num_max_caracteres_username)));
        }else if(!newUsername.matches(noWhiteSpace)){
            userEditText.setError(getResources().getString(R.string.logar_erro_username_whitespace));
        }else{
            userEditText.setError(null);
            userEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }

    private void clearParametersErrors(){
        csfEditText.setError(null);
        csfEditText.setErrorEnabled(false);
        cstEditText.setError(null);
        cstEditText.setErrorEnabled(false);
        csdaEditText.setError(null);
        csdaEditText.setErrorEnabled(false);
        csdcEditText.setError(null);
        csdcEditText.setErrorEnabled(false);
        porcCompEditText.setError(null);
        porcCompEditText.setErrorEnabled(false);
    }
}