package com.example.dams;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.SwitchCompat;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.material.switchmaterial.SwitchMaterial;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link FlutuacaoFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class FlutuacaoFragment extends Fragment {

    private Button simularButton;
    private String feedback;
    private EditText subpressaoEditText, cgxSubpressaoEditText, csfEditText;
    private SwitchCompat subpressaoSwitch;
    private TextView csfMinTextView;
    private SimulacaoViewModel simulacaoViewModel;
    private Number subpressaoU, cgxsubpressaoU, csfU;
    private NumberFormat format;
    private double csfMin;
    private boolean msubpressaoU;

    public FlutuacaoFragment() {
        // Required empty public constructor
    }

    // TODO: Rename and change types and number of parameters
    public static FlutuacaoFragment newInstance() {
        return new FlutuacaoFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        simulacaoViewModel = new ViewModelProvider(getActivity()).get(SimulacaoViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_flutuacao, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        simularButton = getView().findViewById(R.id.flutuacaoButton);
        subpressaoEditText = getView().findViewById(R.id.flutuacaoSubpressaoEditText);
        cgxSubpressaoEditText = getView().findViewById(R.id.flutuacaoCGXSubpressaoEditText);
        csfEditText = getView().findViewById(R.id.flutuacaoCSFEditText);
        subpressaoSwitch = getView().findViewById(R.id.flutuacaoSubpressaoSwitch);
        csfMinTextView = getView().findViewById(R.id.flutuacaoCSFminTextView);

        format = NumberFormat.getInstance(Locale.getDefault());
        try {
            csfMin = format.parse(simulacaoViewModel.getSimulacao().getValue().getCmin().get(0)).doubleValue();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        csfMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csfMin));
        csfMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
        simulacaoViewModel.getSimulacao().getValue().simular();
        listeners();
    }

    private void listeners() {
        subpressaoEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_aviso_unidade),
                            v.getContext().getResources().getString(R.string.simular_flutuacao_subpressao_hint), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(3));
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

        cgxSubpressaoEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_aviso_unidade),
                            v.getContext().getResources().getString(R.string.simular_cgx_hint), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(0));
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

        csfEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                try{
                    if(format.parse(csfEditText.getText().toString()).doubleValue() < csfMin){
                        csfMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csfMin));
                        csfMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                    }else if(format.parse(csfEditText.getText().toString()).doubleValue() == csfMin){
                        csfMinTextView.setText(String.format(Locale.getDefault(),"= %.2f",csfMin));
                        csfMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }else{
                        csfMinTextView.setText(String.format(Locale.getDefault(),"> %.2f",csfMin));
                        csfMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }
                }catch (NumberFormatException e){
                    csfMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csfMin));
                    csfMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable s) {
            }
        });

        simularButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try{
                    subpressaoU = format.parse(subpressaoEditText.getText().toString());
                    cgxsubpressaoU = format.parse(cgxSubpressaoEditText.getText().toString());
                    csfU = format.parse(csfEditText.getText().toString());
                }catch (Exception e){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.aviso_campos_nao_preenchidos));
                }
                msubpressaoU = subpressaoSwitch.isChecked();
                simulacaoViewModel.getSimulacao().getValue().obterFeedbackFlutuacao(subpressaoU.doubleValue(), cgxsubpressaoU.doubleValue(), msubpressaoU, csfU.doubleValue());
                simulacaoViewModel.setSimulacao(simulacaoViewModel.getSimulacao().getValue());
                feedback="";

                if(!feedback.isEmpty()){
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

    }
}