package com.example.dams;

import android.app.ActivityOptions;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.os.Bundle;
import android.util.Pair;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import com.bumptech.glide.Glide;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;

public class IniciarActivity extends AppCompatActivity {

    private ImageView damsLogo;
    private FirebaseAuth firebaseAuth;
    private Button logarButton;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_iniciar);
        logarButton = findViewById(R.id.iniciarLogarButton);
        damsLogo = findViewById(R.id.damsLogoImageView);
        Glide.with(this).load(R.drawable.dams_logo).into(damsLogo);

        firebaseAuth = FirebaseAuth.getInstance();
        FirebaseUser firebaseUser = firebaseAuth.getCurrentUser();

        if(!webConnected()){
            Toast.makeText(this, getResources().getString(R.string.iniciar_sem_conexao), Toast.LENGTH_LONG).show();
            this.finishAffinity();
        }else{
            if(firebaseUser != null){
                Intent it = new Intent(this, MainActivity.class);
                it.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                Bundle params = new Bundle();
                params.putBoolean("hasFeedback", true);
                params.putString("feedback", getResources().getString(R.string.mascote_retorno));
                params.putInt("figmascote", R.drawable.mascote_5);
                params.putInt("animacao", 0);
                it.putExtras(params);
                startActivity(it);
            }
        }
    }

    private boolean webConnected() {
        boolean connected = false;
        ConnectivityManager connectivityManager = (ConnectivityManager)getSystemService(Context.CONNECTIVITY_SERVICE);
        if(connectivityManager.getNetworkInfo(ConnectivityManager.TYPE_MOBILE).getState() == NetworkInfo.State.CONNECTED ||
                connectivityManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI).getState() == NetworkInfo.State.CONNECTED) {
            //we are connected to a network
            connected = true;
        }else{
            connected = false;
        }
            return connected;
    }

    @Override
    protected void onStart() {
        super.onStart();

        logarButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View view){
                Intent it = new Intent(IniciarActivity.this, LogarActivity.class);
                Bundle params = new Bundle();
                params.putBoolean("hasFeedback", true);
                params.putString("feedback", view.getContext().getResources().getString(R.string.mascote_boas_vindas_1));
                params.putInt("figmascote", R.drawable.mascote_5);
                params.putInt("animacao", 0);
                it.putExtras(params);
                if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP){
                    Pair[] pairs = new Pair[1];
                    pairs[0] = new Pair<View,String>(findViewById(R.id.iniciarLogarButton), "transition_login");
                    ActivityOptions options = ActivityOptions.makeSceneTransitionAnimation(IniciarActivity.this, pairs);
                    startActivity(it, options.toBundle());
                }else{
                    startActivity(it);
                }
            }
        });
    }

}