package com.example.dams;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInAccount;
import com.google.android.gms.auth.api.signin.GoogleSignInClient;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.gms.common.SignInButton;
import com.google.android.gms.common.api.ApiException;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.material.textfield.TextInputLayout;
import com.google.firebase.auth.AuthCredential;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.auth.GoogleAuthProvider;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LogarActivity extends AppCompatActivity {

    private static final Pattern VALID_EMAIL_ADDRESS_REGEX = Pattern.compile("^[A-Z0-9._%+-]+@[A-Z0-9.-]+\\.[A-Z]{2,6}$", Pattern.CASE_INSENSITIVE);
    private SignInButton logarComGoogleButton;
    private GoogleSignInClient googleClient;
    private FirebaseAuth firebaseAuth;
    private FirebaseDatabase firebaseDatabase;
    private DatabaseReference databaseReference;
    private Button efetuarLoginButton, recuperarSenhaButton, cadastrarButton;
    private TextInputLayout emailEditText, senhaEditText;
    private ProgressBar progressBar;
    private int loginMethod, figmascote, animacao;
    private String email, senha, feedback;
    private Fragment feedbackFragment;
    private FrameLayout feedbackFrame;
    private boolean hasFeedback = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_logar);
        logarComGoogleButton = findViewById(R.id.logarComGoogleButton);
        efetuarLoginButton = findViewById(R.id.logarButton);
        cadastrarButton = findViewById(R.id.logarCadastrarButton);
        recuperarSenhaButton = findViewById(R.id.logarRecuperarSenhaButton);
        emailEditText = findViewById(R.id.logarEmailEditText);
        senhaEditText = findViewById(R.id.logarSenhaEditText);
        progressBar = findViewById(R.id.logarProgressBar);
        feedbackFrame = findViewById(R.id.logarFeedbackContainer);

        firebaseDatabase = FirebaseDatabase.getInstance();
        databaseReference = firebaseDatabase.getReference().child("users");
        firebaseAuth = FirebaseAuth.getInstance();
        // Configure Google Sign In
        GoogleSignInOptions gso = new GoogleSignInOptions
                .Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                .requestIdToken(getString(R.string.default_web_client_id))
                .requestEmail()
                .build();
        googleClient = GoogleSignIn.getClient(this, gso);
        listeners();
        Intent it = getIntent();
        if(it!=null){
            Bundle params = it.getExtras();
            if(params!=null){
                hasFeedback = params.getBoolean("hasFeedback");
                if(hasFeedback){
                    feedback = params.getString("feedback");
                    figmascote = params.getInt("figmascote");
                    animacao = params.getInt("animacao");
                    showFeedbackFragment();
                }
            }
        }

    }

    public void showFeedbackFragment(){
        if(feedbackFrame.getVisibility() == View.INVISIBLE){
            feedbackFrame.setVisibility(View.VISIBLE);
            FragmentTransaction transaction = getSupportFragmentManager().beginTransaction(); //Gerencia fragmentos
            feedbackFragment = FeedbackFragment.newInstance(feedback,figmascote,animacao);
            transaction.replace(R.id.logarFeedbackContainer, feedbackFragment);
            //transaction.addToBackStack(null);
            transaction.commit();
        }
    }

    private void listeners() {
        feedbackFrame.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                FragmentTransaction transaction = getSupportFragmentManager().beginTransaction(); //Gerencia fragmentos
                transaction.remove(feedbackFragment).commit();
                feedbackFrame.setVisibility(View.INVISIBLE);
                if(feedback.equals(getResources().getString(R.string.mascote_boas_vindas_1))){
                    feedback = getResources().getString(R.string.mascote_boas_vindas_2);
                    figmascote = R.drawable.mascote_11;
                    showFeedbackFragment();
                }
                }
            });

        efetuarLoginButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View view){
                loginMethod = 0;
                logarComEmail();
            }
        });

        logarComGoogleButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                loginMethod = 1;
                Intent it = googleClient.getSignInIntent();
                startActivityForResult(it, 100);
            }
        });

        cadastrarButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View view){
                Intent it = new Intent(LogarActivity.this, CadastrarActivity.class);
                startActivity(it);
            }
        });

        recuperarSenhaButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View view){
                Intent it = new Intent(LogarActivity.this, RecuperarSenhaActivity.class);
                startActivity(it);
            }
        });
    }

    private void logarComEmail() {
        if(validarEmail() && validarSenha()){
            progressBar.setVisibility(View.VISIBLE);
            firebaseAuth.signInWithEmailAndPassword(email,senha).addOnCompleteListener(new OnCompleteListener<AuthResult>() {
                @Override
                public void onComplete(@NonNull Task<AuthResult> task) {
                    progressBar.setVisibility(View.INVISIBLE);
                    if(task.isSuccessful()){
                        Toast.makeText(LogarActivity.this, getResources().getString(R.string.logar_email_sucesso), Toast.LENGTH_LONG).show();
                        Intent it = new Intent(LogarActivity.this, MainActivity.class);
                        Bundle params = new Bundle();
                        params.putBoolean("isLogin", true);
                        params.putBoolean("hasFeedback", true);
                        params.putString("feedback", getResources().getString(R.string.mascote_recepcao));
                        params.putInt("figmascote", R.drawable.mascote_5);
                        params.putInt("animacao", 0);
                        it.putExtras(params);
                        it.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                        startActivity(it);
                    }else{
                        Toast.makeText(LogarActivity.this, task.getException().getMessage(), Toast.LENGTH_LONG).show();
                    }
                }
            });
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode == 100){
            progressBar.setVisibility(View.VISIBLE);
            switch (loginMethod){
                case 1:{
                    Task<GoogleSignInAccount> signInAccountTask = GoogleSignIn.getSignedInAccountFromIntent(data);
                    if(signInAccountTask.isSuccessful()){
                        //Toast.makeText(this, this.getResources().getString(R.string.logar_google_sucesso), Toast.LENGTH_LONG).show();
                        try {
                            GoogleSignInAccount googleSignInAccount = signInAccountTask.getResult(ApiException.class);
                            if(googleSignInAccount != null){
                                AuthCredential authCredential = GoogleAuthProvider.getCredential(googleSignInAccount.getIdToken(),null);
                                firebaseAuth.signInWithCredential(authCredential).addOnCompleteListener(new OnCompleteListener<AuthResult>() {
                                    @Override
                                    public void onComplete(@NonNull Task<AuthResult> task) {
                                        if(task.isSuccessful()){
                                            databaseReference.child(firebaseAuth.getUid()).addListenerForSingleValueEvent(new ValueEventListener() {
                                                @Override
                                                public void onDataChange(@NonNull DataSnapshot snapshot) {
                                                    if(snapshot.getValue(Usuario.class) == null){
                                                        FirebaseUser firebaseUser = firebaseAuth.getCurrentUser();
                                                        String username = firebaseUser.getDisplayName().replaceAll("\\s+","");
                                                        if(username.length() > getResources().getInteger(R.integer.num_max_caracteres_username)){
                                                            username = username.substring(0,getResources().getInteger(R.integer.num_max_caracteres_username));
                                                        }
                                                        Usuario usuario = new Usuario(loginMethod, username);
                                                        usuario.setParametrosPadrao(LogarActivity.this);
                                                        databaseReference.child(firebaseAuth.getUid()).setValue(usuario); //New User
                                                    }
                                                    Intent it = new Intent(LogarActivity.this, MainActivity.class);
                                                    Bundle params = new Bundle();
                                                    params.putBoolean("isLogin", true);
                                                    params.putBoolean("hasFeedback", true);
                                                    params.putString("feedback", getResources().getString(R.string.mascote_recepcao));
                                                    params.putInt("figmascote", R.drawable.mascote_5);
                                                    params.putInt("animacao", 0);
                                                    it.putExtras(params);
                                                    it.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                                                    startActivity(it);
                                                    Toast.makeText(LogarActivity.this, LogarActivity.this.getResources().getString(R.string.logar_google_sucesso), Toast.LENGTH_LONG).show();
                                                    progressBar.setVisibility(View.INVISIBLE);
                                                }
                                                @Override
                                                public void onCancelled(@NonNull DatabaseError error) {
                                                    Toast.makeText(LogarActivity.this,getResources().getString(R.string.erro_bd),Toast.LENGTH_LONG).show();
                                                    progressBar.setVisibility(View.INVISIBLE);
                                                }
                                            });
                                        }else{
                                            Toast.makeText(LogarActivity.this, LogarActivity.this.getResources().getString(R.string.logar_google_falhou)+task.getException().getMessage(), Toast.LENGTH_LONG).show();
                                            progressBar.setVisibility(View.INVISIBLE);
                                        }
                                    }
                                });
                            }
                        } catch (ApiException e) {
                            e.printStackTrace();
                        }
                    }
                    break;
                }
                case 2:{
                    //Login com Facebook
                }
            }

        }
    }

    public boolean validarSenha(){
        senha = senhaEditText.getEditText().getText().toString();
        boolean valido = false;
        if(senha.isEmpty()){
            senhaEditText.setError(getResources().getString(R.string.logar_erro_senha_vazia));
        }else{
            senhaEditText.setError(null);
            senhaEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }

    public boolean validarEmail() {
        email = emailEditText.getEditText().getText().toString().trim();
        Matcher matcher = VALID_EMAIL_ADDRESS_REGEX.matcher(email);
        boolean padronizado = matcher.find();
        boolean valido = false;
        if(email.isEmpty()){
            emailEditText.setError(getResources().getString(R.string.cadastrar_erro_email_vazio));
        }else if(!padronizado){
            emailEditText.setError(getResources().getString(R.string.cadastrar_erro_email_invalido));
        }else{
            emailEditText.setError(null);
            emailEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }

    /*public boolean validarUsuario(){
        user = userEditText.getEditText().getText().toString().trim();
        boolean valido = false;
        if(validarEmail(user)){
            userEditText.setError(null);
            valido = true;
        }else{
            String noWhiteSpace = "\\A\\w{6,20}\\z";
            if(user.length() < getResources().getInteger(R.integer.num_min_caracteres_username)){
                userEditText.setError(String.format(Locale.getDefault(), getResources().getString(R.string.logar_erro_username_curto), getResources().getInteger(R.integer.num_min_caracteres_username)));
            }else if(user.length() > getResources().getInteger(R.integer.num_max_caracteres_username)){
                userEditText.setError(String.format(Locale.getDefault(), getResources().getString(R.string.logar_erro_username_longo), getResources().getInteger(R.integer.num_max_caracteres_username)));
            }else if(!user.matches(noWhiteSpace)){
                userEditText.setError(getResources().getString(R.string.logar_erro_username_whitespace));
            }else{
                userEditText.setError(null);
                userEditText.setErrorEnabled(false);
                valido = true;
            }
        }
        return valido;
    }*/
}