package com.example.dams;

import android.app.ActivityOptions;
import android.content.Intent;
import android.content.res.TypedArray;
import android.os.Build;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.res.ResourcesCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

import smartdevelop.ir.eram.showcaseviewlib.GuideView;
import smartdevelop.ir.eram.showcaseviewlib.config.DismissType;
import smartdevelop.ir.eram.showcaseviewlib.config.Gravity;
import smartdevelop.ir.eram.showcaseviewlib.listener.GuideListener;

public class MainActivity extends AppCompatActivity implements BottomNavigationView.OnNavigationItemSelectedListener {

    private ProgressBar progressBar;
    private BottomNavigationView menu;
    private List<String> titulosIntro, descricoesIntro;
    private List<Integer> idsIntro;
    private boolean isLogin = false, hasFeedback = false, hasUpdate = false;
    private UsuarioViewModel usuarioViewModel;
    private DatabaseReference databaseReference;
    private Fragment feedbackFragment;
    private FrameLayout feedbackFrame;
    private String feedback;
    private int figmascote, animacao, posModulo, posCapitulo;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        progressBar = findViewById(R.id.mainProgressBar);
        feedbackFrame = findViewById(R.id.mainFeedbackContainer);
        menu = findViewById(R.id.mainMenuBottomView);

        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP){
            menu.setItemBackground(ResourcesCompat.getDrawable(getResources(), R.drawable.ripple_effect, null));
        }
        menu.setOnNavigationItemSelectedListener(this);
        menu.setSelectedItemId(0);
        FirebaseAuth firebaseAuth = FirebaseAuth.getInstance();
        databaseReference = FirebaseDatabase.getInstance().getReference().child("users").child(firebaseAuth.getUid());
        progressBar.setVisibility(View.VISIBLE);
        Intent it = getIntent();
        if(it!=null){
            Bundle params = it.getExtras();
            if(params!=null){
                isLogin = params.getBoolean("isLogin");
                hasFeedback = params.getBoolean("hasFeedback");
                hasUpdate = params.getBoolean("hasUpdate");
                if(hasFeedback){
                    feedback = params.getString("feedback");
                    figmascote = params.getInt("figmascote");
                    animacao = params.getInt("animacao");
                }
                if(hasUpdate){
                    posModulo = params.getInt("posModulo");
                    posCapitulo = params.getInt("posCapitulo");
                    atualizarProgresso();
                }
            }
        }

        usuarioViewModel = new ViewModelProvider(this).get(UsuarioViewModel.class);
        databaseReference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot snapshot) {
                usuarioViewModel.setUsuario(snapshot.getValue(Usuario.class));
                progressBar.setVisibility(View.INVISIBLE);
                if(savedInstanceState == null){
                    Fragment modulosFragment = ModulosFragment.newInstance();
                    openFragment(modulosFragment);
                }
                listeners();
                loginIntro();
            }
            @Override
            public void onCancelled(@NonNull DatabaseError error) {
                Toast.makeText(MainActivity.this,getResources().getString(R.string.erro_bd),Toast.LENGTH_LONG).show();
                progressBar.setVisibility(View.INVISIBLE);
            }
        });
        if(!isLogin){
            showFeedbackFragment();
        }
    }

    private void atualizarProgresso() {
        databaseReference.child("progressoCurso").child(Integer.toString(posModulo)).child(Integer.toString(posCapitulo)).setValue(1).addOnCompleteListener(new OnCompleteListener<Void>() {
            @Override
            public void onComplete(@NonNull Task<Void> task) {
                if(!task.isSuccessful()){
                    Toast.makeText(MainActivity.this,getResources().getString(R.string.erro_bd),Toast.LENGTH_LONG).show();
                }
            }
        });
    }

    public void showFeedbackFragment(){
        if(feedbackFrame.getVisibility() == View.INVISIBLE){
            feedbackFrame.setVisibility(View.VISIBLE);
            FragmentTransaction transaction = getSupportFragmentManager().beginTransaction(); //Gerencia fragmentos
            feedbackFragment = FeedbackFragment.newInstance(feedback,figmascote,animacao);
            transaction.replace(R.id.mainFeedbackContainer, feedbackFragment);
            //transaction.addToBackStack(null);
            transaction.commit();
        }
    }

    private void listeners() {
        feedbackFrame.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                FragmentTransaction transaction = getSupportFragmentManager().beginTransaction(); //Gerencia fragmentos
                transaction.remove(feedbackFragment).commit();
                feedbackFrame.setVisibility(View.INVISIBLE);
            }
        });
    }

    private void loginIntro() {
        if(isLogin){
            titulosIntro = new ArrayList<>();
            descricoesIntro = new ArrayList<>();
            idsIntro = new ArrayList<>();
            titulosIntro.addAll(Arrays.asList(getResources().getStringArray(R.array.modulos_intro_titulos)));
            descricoesIntro.addAll(Arrays.asList(getResources().getStringArray(R.array.modulos_intro_descricoes)));
            TypedArray typedArray = getResources().obtainTypedArray(R.array.modulos_intro_ids);
            for(int ind=0; ind<typedArray.length(); ind++){
                idsIntro.add(typedArray.getResourceId(ind,0));
            }
            typedArray.recycle();
            ShowIntro(titulosIntro.get(0), descricoesIntro.get(0), idsIntro.get(0), 0);
        }
    }

    @Override
    public boolean onNavigationItemSelected(@NonNull MenuItem item) {
        switch (item.getItemId()) {
            case R.id.navigation_modulos: {
                Fragment modulosFragment = ModulosFragment.newInstance();
                openFragment(modulosFragment);
                break;
            }
            /*case R.id.navigation_desafios: {
                Fragment desafiosFragment = DesafiosFragment.newInstance();
                openFragment(desafiosFragment);
                break;
            }*/
            case R.id.navigation_dados: {
                Fragment dadosFragment = DadosFragment.newInstance();
                openFragment(dadosFragment);
                break;
            }
        }
        return true;
    }
    private void openFragment(Fragment fragment) {
        FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
        transaction.replace(R.id.mainContainer, fragment);
        //transaction.addToBackStack(null);
        transaction.commit();
    }

    private void ShowIntro(String title, String text, int viewId, final int type) {

        new GuideView.Builder(this)
                .setTitle(title)
                .setContentText(text)
                .setGravity(Gravity.auto)
                .setTargetView(findViewById(viewId))
                .setContentTextSize(12)//optional
                .setTitleTextSize(14)//optional
                .setDismissType(DismissType.anywhere) //optional - default dismissible by TargetView
                .setGuideListener(new GuideListener() {
                    @Override
                    public void onDismiss(View view) {
                        if(type < idsIntro.size()-1){
                            ShowIntro(titulosIntro.get(type + 1), descricoesIntro.get(type + 1), idsIntro.get(type + 1), type + 1);
                        }else{
                            showFeedbackFragment();
                            //SharePrefUtils.putBoolean("showcase", false);
                        }
                    }
                })
                .build()
                .show();
    }

}
