package com.example.dams;

import android.media.MediaPlayer;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.VideoView;

import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import pl.droidsonroids.gif.GifImageView;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link ModulosFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class ModulosFragment extends Fragment {

    private GifImageView progressImageView;
    private RecyclerView modulosRV;
    private List<String> modulos;
    private ModulosRVAdapter modulosAdapter;
    private UsuarioViewModel usuarioViewModel;

    public ModulosFragment() {
        // Required empty public constructor
    }

    public static ModulosFragment newInstance() {
        return new ModulosFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //Fragment Módulos
        usuarioViewModel = new ViewModelProvider(getActivity()).get(UsuarioViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_modulos, container, false);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        modulosRV = getView().findViewById(R.id.modulosRecyclerView);
        modulosRV.setNestedScrollingEnabled(false);
        progressImageView = getView().findViewById(R.id.progressImageView);
        modulos = new ArrayList<>();
        modulos.addAll(Arrays.asList(getResources().getStringArray(R.array.modulos)));
        modulosAdapter = new ModulosRVAdapter(ModulosFragment.this.getContext(), modulos, usuarioViewModel.getUsuario().getValue());
        LinearLayoutManager layoutManager = new LinearLayoutManager(getView().getContext());
        layoutManager.setOrientation(RecyclerView.VERTICAL);
        DividerItemDecoration dividerItemDecoration = new DividerItemDecoration(modulosRV.getContext(), layoutManager.getOrientation());
        modulosRV.addItemDecoration(dividerItemDecoration);
        modulosRV.setLayoutManager(layoutManager);
        modulosRV.setAdapter(modulosAdapter);
        setLevel();

    }

    private void setLevel() {
        List<List<Integer>> progressoCurso = usuarioViewModel.getUsuario().getValue().getProgressoCurso();
        int level = 0;
        int numCapCompletos;
        for(int i=0; i<progressoCurso.size(); i++){
            numCapCompletos = 0;
            for(int j=0; j<progressoCurso.get(i).size();j++){
                if(progressoCurso.get(i).get(j) == 1){
                    numCapCompletos = numCapCompletos + 1;
                }
            }
            if(progressoCurso.get(i).size() == numCapCompletos){
                level = level + 1;
            }
        }
        int animacao = 0;
        switch(level){
            case 0:{
                progressImageView.setImageResource(R.drawable.dam1);
                break;
            }
            case 1:{
                progressImageView.setImageResource(R.drawable.dam2);
                break;
            }
            case 2:{
                progressImageView.setImageResource(R.drawable.dam3);
                break;
            }
            case 3:{
                progressImageView.setImageResource(R.drawable.dam4);
                break;
            }
            case 4:{
                progressImageView.setImageResource(R.drawable.dam5);
                break;
            }
            case 5:{
                progressImageView.setImageResource(R.drawable.dam6);
                break;
            }
            case 6:{
                progressImageView.setImageResource(R.drawable.dam7);
                break;
            }
            case 7:{
                progressImageView.setImageResource(R.drawable.dam8);
                break;
            }
        }

    }

}