package com.example.dams;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

public class ModulosRVAdapter extends RecyclerView.Adapter<ModulosRVAdapter.ViewHolder>{

    private Context context;
    private List<String> modulos;
    private CapitulosRVAdapter capitulosRVAdapter;
    private Usuario usuario;
    private Modulo modulo;

    public ModulosRVAdapter(Context context, List<String> modulos, Usuario usuario) {
        this.context = context;
        this.modulos = modulos;
        this.usuario = usuario;
    }

    @NonNull
    @Override
    public ModulosRVAdapter.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.modulos_rv_row, parent, false);
        return new ModulosRVAdapter.ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull ModulosRVAdapter.ViewHolder holder, int position) {
        String nome = modulos.get(position);
        holder.moduloTituloTextView.setText(nome);
        List<String> cap = new ArrayList<>();
        List<String> tipoCapitulos = new ArrayList<>();
        switch (position){
            case 0: {
                cap.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo0_capitulos))); //add capitulos
                tipoCapitulos.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo0_tipos_capitulos))); //add tipoCapitulos
                break;
            }
            case 1: {
                cap.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo1_capitulos))); //add capitulos
                tipoCapitulos.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo1_tipos_capitulos))); //add tipoCapitulos
                break;
            }
            case 2: {
                cap.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo2_capitulos))); //add capitulos
                tipoCapitulos.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo2_tipos_capitulos))); //add tipoCapitulos
                break;
            }
            case 3: {
                cap.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo3_capitulos))); //add capitulos
                tipoCapitulos.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo3_tipos_capitulos))); //add tipoCapitulos
                break;
            }
            case 4: {
                cap.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo4_capitulos))); //add capitulos
                tipoCapitulos.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo4_tipos_capitulos))); //add tipoCapitulos
                break;
            }
            case 5: {
                cap.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo5_capitulos))); //add capitulos
                tipoCapitulos.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo5_tipos_capitulos))); //add tipoCapitulos
                break;
            }
            case 6: {
                cap.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo6_capitulos))); //add capitulos
                tipoCapitulos.addAll(Arrays.asList(context.getResources().getStringArray(R.array.modulo6_tipos_capitulos))); //add tipoCapitulos
                break;
            }
        }
        List<Capitulo> capitulos = new ArrayList<>();
        for(int i = 0; i < cap.size(); i++){
            Capitulo capitulo = new Capitulo(cap.get(i));
            capitulos.add(capitulo);
        }
        int progresso = calcularProgressoModulo(usuario.getProgressoCurso().get(position));
        holder.moduloProgressBar.setProgress(progresso);
        holder.moduloProgressTextView.setText(String.format(Locale.getDefault(), "%d%s", progresso, "%"));
        modulo = new Modulo(position, nome, capitulos, progresso);
        capitulosRVAdapter = new CapitulosRVAdapter(context, modulo, tipoCapitulos, usuario);
        //capitulosRVAdapter.setClickListener(this);
        LinearLayoutManager capitulosLayout = new LinearLayoutManager(context);
        holder.capitulosModuloRV.setLayoutManager(capitulosLayout);
        holder.capitulosModuloRV.setAdapter(capitulosRVAdapter);
    }

    private int calcularProgressoModulo(List<Integer> progressoModulo) {
        int soma = 0;
        for(int i = 0; i < progressoModulo.size(); i++){
            soma = soma + progressoModulo.get(i);
        }
        return (int) Math.round(100 * (double) soma / (double) progressoModulo.size());
    }

    @Override
    public int getItemCount() {
        return modulos.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        TextView moduloTituloTextView, moduloProgressTextView;
        ProgressBar moduloProgressBar;
        RecyclerView capitulosModuloRV;
        //CardView cardModulo;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            moduloTituloTextView = itemView.findViewById(R.id.moduloTextView);
            moduloProgressBar = itemView.findViewById(R.id.moduloProgressBar);
            moduloProgressTextView = itemView.findViewById(R.id.moduloProgressTextView);
            capitulosModuloRV = itemView.findViewById(R.id.capitulosRecyclerView);
            //cardModulo = itemView.findViewById(R.id.moduloCardView);
        }
    }

    /*@Override
    public void onItemClick(View view, int position) {
        Toast.makeText(context, "You clicked " + capitulosRVAdapter.getItem(position) + " on row number " + position, Toast.LENGTH_SHORT).show();
    }*/
}
