package com.example.dams;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.material.textfield.TextInputLayout;
import com.google.firebase.auth.FirebaseAuth;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class RecuperarSenhaActivity extends AppCompatActivity {

    private static final Pattern VALID_EMAIL_ADDRESS_REGEX = Pattern.compile("^[A-Z0-9._%+-]+@[A-Z0-9.-]+\\.[A-Z]{2,6}$", Pattern.CASE_INSENSITIVE);
    private FirebaseAuth firebaseAuth;
    private ProgressBar progressBar;
    private Button recuperarSenhaButton;
    private TextInputLayout emailEditText;
    private String email;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_recuperar_senha);
        recuperarSenhaButton = findViewById(R.id.recuperarSenhaButton);
        emailEditText = findViewById(R.id.recuperarSenhaEmailEditText);
        progressBar = findViewById(R.id.recuperarSenhaProgressBar);
        firebaseAuth = FirebaseAuth.getInstance();
        listerners();
    }

    private void listerners() {
        recuperarSenhaButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View view){
                if(validarEmail()){
                    progressBar.setVisibility(View.VISIBLE);
                    firebaseAuth.sendPasswordResetEmail(email).addOnCompleteListener(new OnCompleteListener<Void>() {
                        @Override
                        public void onComplete(@NonNull Task<Void> task) {
                            if(task.isSuccessful()){
                                Toast.makeText(view.getContext(), getResources().getString(R.string.recuperarSenha_sucesso), Toast.LENGTH_LONG).show();
                                Intent it = new Intent(RecuperarSenhaActivity.this, LogarActivity.class);
                                startActivity(it);
                            }else{
                                Toast.makeText(view.getContext(), getResources().getString(R.string.recuperarSenha_falhou), Toast.LENGTH_LONG).show();
                                progressBar.setVisibility(View.INVISIBLE);
                            }
                        }
                    });
                }
            }
        });
    }

    public boolean validarEmail() {
        email = emailEditText.getEditText().getText().toString().trim();
        Matcher matcher = VALID_EMAIL_ADDRESS_REGEX.matcher(email);
        boolean padronizado = matcher.find();
        boolean valido = false;
        if(email.isEmpty()){
            emailEditText.setError(getResources().getString(R.string.cadastrar_erro_email_vazio));
        }else if(!padronizado){
            emailEditText.setError(getResources().getString(R.string.cadastrar_erro_email_invalido));
        }else{
            emailEditText.setError(null);
            emailEditText.setErrorEnabled(false);
            valido = true;
        }
        return valido;
    }
}