package com.example.dams;

import android.content.Context;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

public abstract class Simulacao extends Capitulo{
    protected static final NumberFormat format = NumberFormat.getInstance(Locale.getDefault());
    protected boolean fim;
    protected String feedback;
    protected int cc, figmascote, animacao, posModulo, posCapitulo;
    protected List<String> cmin, unidades;
    protected double tolerancia, a, b, h, gamac, peso, cgx, cgy, m, j, gamaa, eam, cgyeam, eaj, cgyeaj, pesoaj, cgxpesoaj, subpressao, cgxsubpressao, csf, somamest, somamdes, cst, somafh, somafv, fi, csda, coesaominnec, coesaomax, csdc, tensaomax, tensaomin, porcbasecomp, fissura;

    public Simulacao() {
    }

    protected Simulacao(int posModulo, int posCapitulo, String titulo, double a, double b, double h, double gamac) {
        super(titulo);
        this.posModulo = posModulo;
        this.posCapitulo = posCapitulo;
        this.a = a;
        this.b = b;
        this.h = h;
        this.gamac = gamac;
    }

    public abstract void simular();
    public abstract void simularDeslizamentoCoesao();
    public abstract void simularFissura();
    public abstract void calcularPeso();
    public abstract void calcularCGX();
    public abstract void calcularCGY();
    public abstract void calcularEAM();
    public abstract void calcularCGYEAM();
    public abstract void calcularEAJ();
    public abstract void calcularCGYEAJ();
    public abstract void calcularPesoAJ();
    public abstract void calcularCGXPesoAJ();
    public abstract void calcularSubpressao();
    public abstract void calcularCGXSubpressao();
    public abstract void calcularCSF();
    public abstract void calcularSomaMEst();
    public abstract void calcularSomaMDes();
    public abstract void calcularCST();
    public abstract void calcularSomaFH();
    public abstract void calcularSomaFV();
    public abstract void calcularCSDa();
    public abstract void calcularCoesaoMinNec();
    public abstract void calcularCSDc();
    public abstract void calcularTensaoMax();
    public abstract void calcularTensaoMin();
    public abstract void calcularPorcBaseComp();
    public abstract void calcularFissura();
    public abstract void calcularNovosCoefTenPorc();
    public abstract void obterFeedbackCentroMassa(double pesoU, double cgxU, double cgyU);
    public abstract void obterFeedbackForcaPressao(double eamU, double cgyeamU, boolean meamU, double eajU, double cgyeajU, boolean meajU, double pesoajU, double cgxpesoajU, boolean mpesoajU, double pesoamU, double cgxpesoamU, boolean mpesoamU);
    public abstract void obterFeedbackFlutuacao(double subpressaoU, double cgxsubpressaoU, boolean msubpressaoU, double csfU);
    public abstract void obterFeedbackTombamento(double somamestU, double somamdesU, double cstU);
    public abstract boolean obterFeedbackDeslizamentoAtrito(double somafhU, double csdaU);
    public abstract void obterFeedbackDeslizamentoCoesao(double somafvU, double coesaominnecU, double csdcU);
    public abstract boolean obterFeedbackTensaoBase(double tensaomaxU, double tensaominU, double porcbasecompU);
    public abstract void obterFeedbackFissura(double csfU, double cstU, double csdaU, double csdcU, double tensaominU, double porcbasecompU);
    public double getC(){return 0;};
    public void setC(double c){};
    public double getD(){return 0;};
    public void setD(double d){};
    public double getPesoam(){return 0;};
    public void setPesoam(double pesoam){};
    public double getCgxpesoam(){return 0;};
    public void setCgxpesoam(double cgxpesoam){};

    public double parseDouble(String string){
        double number = 0;
        try {
            number = format.parse(string).doubleValue();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return number;
    }

    public boolean acertouDouble(double valorCorreto, double valorUsuario){
        boolean acertou = false;
        if(valorCorreto >= 0){
            if((valorCorreto - tolerancia * valorCorreto) <= valorUsuario && valorUsuario <= (valorCorreto + tolerancia * valorCorreto)){
                acertou = true;
            }
        }else{
            if((valorCorreto - tolerancia * valorCorreto) >= valorUsuario && valorUsuario >= (valorCorreto + tolerancia * valorCorreto)){
                acertou = true;
            }
        }
        return acertou;
    }

    public void obterCMIN(Context context) { //Implementar em CasoCarregamentoActivity
        this.cmin = new ArrayList<String>();
        switch (cc) {
            case 0: { //CCN
                this.cmin.addAll(Arrays.asList(context.getResources().getStringArray(R.array.ccn_eletrobras)));
                break;
            }
            case 1: { //CCE
                this.cmin.addAll(Arrays.asList(context.getResources().getStringArray(R.array.cce_eletrobras)));
                break;
            }
            case 2: { //CCL
                this.cmin.addAll(Arrays.asList(context.getResources().getStringArray(R.array.ccl_eletrobras)));
                break;
            }
            case 3: { //CCC
                this.cmin.addAll(Arrays.asList(context.getResources().getStringArray(R.array.ccc_eletrobras)));
                break;
            }
        }
    }

    public void obterUnidades(Context context) { //Implementar em SelecionarSecaoActivity
        this.unidades = new ArrayList<String>();
        this.unidades.addAll(Arrays.asList(context.getResources().getStringArray(R.array.simulacao_unidades_SI)));
    }

    public void obterTolerancia(Context context){
        this.tolerancia = Double.parseDouble(context.getResources().getString(R.string.simulacao_porcentagem_tolerancia));
    }

    public int getPosModulo() {
        return posModulo;
    }

    public void setPosModulo(int posModulo) {
        this.posModulo = posModulo;
    }

    public int getPosCapitulo() {
        return posCapitulo;
    }

    public void setPosCapitulo(int posCapitulo) {
        this.posCapitulo = posCapitulo;
    }

    public boolean isFim() {
        return fim;
    }

    public void setFim(boolean fim) {
        this.fim = fim;
    }

    public double getTolerancia() {
        return tolerancia;
    }

    public void setTolerancia(double tolerancia) {
        this.tolerancia = tolerancia;
    }

    public List<String> getUnidades() {
        return unidades;
    }

    public void setUnidades(List<String> unidades) {
        this.unidades = unidades;
    }

    public List<String> getCmin() {
        return cmin;
    }

    public void setCmin(List<String> cmin) {
        this.cmin = cmin;
    }

    public double getA() {
        return a;
    }

    public void setA(double a) {
        this.a = a;
    }

    public double getB() {
        return b;
    }

    public void setB(double b) {
        this.b = b;
    }

    public double getH() {
        return h;
    }

    public void setH(double h) {
        this.h = h;
    }

    public double getGamac() {
        return gamac;
    }

    public void setGamac(double gamac) {
        this.gamac = gamac;
    }

    public double getPeso() {
        return peso;
    }

    public void setPeso(double peso) {
        this.peso = peso;
    }

    public double getCgx() {
        return cgx;
    }

    public void setCgx(double cgx) {
        this.cgx = cgx;
    }

    public double getCgy() {
        return cgy;
    }

    public void setCgy(double cgy) {
        this.cgy = cgy;
    }

    public double getM() {
        return m;
    }

    public void setM(double m) {
        this.m = m;
    }

    public double getJ() {
        return j;
    }

    public void setJ(double j) {
        this.j = j;
    }

    public double getGamaa() {
        return gamaa;
    }

    public void setGamaa(double gamaa) {
        this.gamaa = gamaa;
    }

    public double getEam() {
        return eam;
    }

    public void setEam(double eam) {
        this.eam = eam;
    }

    public double getCgyeam() {
        return cgyeam;
    }

    public void setCgyeam(double cgyeam) {
        this.cgyeam = cgyeam;
    }

    public double getEaj() {
        return eaj;
    }

    public void setEaj(double eaj) {
        this.eaj = eaj;
    }

    public double getCgyeaj() {
        return cgyeaj;
    }

    public void setCgyeaj(double cgyeaj) {
        this.cgyeaj = cgyeaj;
    }

    public double getPesoaj() {
        return pesoaj;
    }

    public void setPesoaj(double pesoaj) {
        this.pesoaj = pesoaj;
    }

    public double getCgxpesoaj() {
        return cgxpesoaj;
    }

    public void setCgxpesoaj(double cgxpesoaj) {
        this.cgxpesoaj = cgxpesoaj;
    }

    public double getSubpressao() {
        return subpressao;
    }

    public void setSubpressao(double subpressao) {
        this.subpressao = subpressao;
    }

    public double getCgxsubpressao() {
        return cgxsubpressao;
    }

    public void setCgxsubpressao(double cgxsubpressao) {
        this.cgxsubpressao = cgxsubpressao;
    }

    public double getCsf() {
        return csf;
    }

    public void setCsf(double csf) {
        this.csf = csf;
    }

    public double getSomamest() {
        return somamest;
    }

    public void setSomamest(double somamest) {
        this.somamest = somamest;
    }

    public double getSomamdes() {
        return somamdes;
    }

    public void setSomamdes(double somamdes) {
        this.somamdes = somamdes;
    }

    public double getCst() {
        return cst;
    }

    public void setCst(double cst) {
        this.cst = cst;
    }

    public double getSomafh() {
        return somafh;
    }

    public void setSomafh(double somafh) {
        this.somafh = somafh;
    }

    public double getSomafv() {
        return somafv;
    }

    public void setSomafv(double somafv) {
        this.somafv = somafv;
    }

    public double getFi() {
        return fi;
    }

    public void setFi(double fi) {
        this.fi = fi;
    }

    public double getCsda() {
        return csda;
    }

    public void setCsda(double csda) {
        this.csda = csda;
    }

    public double getCoesaominnec() {
        return coesaominnec;
    }

    public void setCoesaominnec(double coesaominnec) {
        this.coesaominnec = coesaominnec;
    }

    public double getCoesaomax() {
        return coesaomax;
    }

    public void setCoesaomax(double coesaomax) {
        this.coesaomax = coesaomax;
    }

    public double getCsdc() {
        return csdc;
    }

    public void setCsdc(double csdc) {
        this.csdc = csdc;
    }

    public double getTensaomax() {
        return tensaomax;
    }

    public void setTensaomax(double tensaomax) {
        this.tensaomax = tensaomax;
    }

    public double getTensaomin() {
        return tensaomin;
    }

    public void setTensaomin(double tensaomin) {
        this.tensaomin = tensaomin;
    }

    public double getPorcbasecomp() {
        return porcbasecomp;
    }

    public void setPorcbasecomp(double porcbasecomp) {
        this.porcbasecomp = porcbasecomp;
    }

    public double getFissura() {
        return fissura;
    }

    public void setFissura(double fissura) {
        this.fissura = fissura;
    }

    public int getCc() {
        return cc;
    }

    public void setCc(int cc) {
        this.cc = cc;
    }

    public String getFeedback() {
        return feedback;
    }

    public void setFeedback(String feedback) {
        this.feedback = feedback;
    }

    public int getFigmascote() {
        return figmascote;
    }

    public void setFigmascote(int figmascote) {
        this.figmascote = figmascote;
    }

    public int getAnimacao() {
        return animacao;
    }

    public void setAnimacao(int animacao) {
        this.animacao = animacao;
    }
}
