package com.example.dams;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.res.ResourcesCompat;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.bumptech.glide.Glide;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link TensoesFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class TensoesFragment extends Fragment {

    private ImageView fissuraImageView;
    private TextView avisoFissuraTextView;
    private Button simularButton;
    private String feedback;
    private EditText tensaoMinEditText, tensaoMaxEditText, porcCompEditText, csfEditText, cstEditText, csdaEditText, csdcEditText, fissuraEditText;
    private TextView porcCompMinTextView, csfMinTextView, cstMinTextView, csdaMinTextView, csdcMinTextView;
    private SimulacaoViewModel simulacaoViewModel;
    private Number tensaomaxU, tensaominU, porcbasecompU, csfU, cstU, csdaU, csdcU;
    private double csfMin, cstMin, csdaMin, csdcMin=1, porcbasecompMin;
    private boolean fissura = false;
    private NumberFormat format;

    public TensoesFragment() {
        // Required empty public constructor
    }

    // TODO: Rename and change types and number of parameters
    public static TensoesFragment newInstance() {
        return new TensoesFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        simulacaoViewModel = new ViewModelProvider(getActivity()).get(SimulacaoViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_tensoes, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        fissuraImageView = getView().findViewById(R.id.fissuraImageView);
        avisoFissuraTextView = getView().findViewById(R.id.fissuraAvisoTextView);
        simularButton = getView().findViewById(R.id.tensoesButton);
        tensaoMinEditText = getView().findViewById(R.id.tensoesMinEditText);
        tensaoMaxEditText = getView().findViewById(R.id.tensoesMaxEditText);
        porcCompEditText = getView().findViewById(R.id.tensoesBaseCompEditText);
        csfEditText = getView().findViewById(R.id.fissuraCSFEditText);
        cstEditText = getView().findViewById(R.id.fissuraCSTEditText);
        csdaEditText = getView().findViewById(R.id.fissuraCSDaEditText);
        csdcEditText = getView().findViewById(R.id.fissuraCSDcEditText);
        fissuraEditText = getView().findViewById(R.id.fissuraEditText);
        porcCompMinTextView = getView().findViewById(R.id.tensoesBaseCompMinTextView);
        csfMinTextView = getView().findViewById(R.id.fissuraCSFTextView);
        cstMinTextView = getView().findViewById(R.id.fissuraCSTTextView);
        csdaMinTextView = getView().findViewById(R.id.fissuraCSDaTextView);
        csdcMinTextView = getView().findViewById(R.id.fissuraCSDcTextView);

        format = NumberFormat.getInstance(Locale.getDefault());
        try {
            csfMin = format.parse(simulacaoViewModel.getSimulacao().getValue().getCmin().get(0)).doubleValue();
            cstMin = format.parse(simulacaoViewModel.getSimulacao().getValue().getCmin().get(1)).doubleValue();
            csdaMin = format.parse(simulacaoViewModel.getSimulacao().getValue().getCmin().get(2)).doubleValue();
            porcbasecompMin = format.parse(simulacaoViewModel.getSimulacao().getValue().getCmin().get(4)).doubleValue();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        fissuraEditText.setText(String.format(Locale.getDefault(),getResources().getString(R.string.simular_parametros_set_text), getResources().getString(R.string.simular_fissura_hint),
                simulacaoViewModel.getSimulacao().getValue().getFissura(), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(0)));
        csfMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csfMin));
        cstMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",cstMin));
        csdaMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csdaMin));
        csdcMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csdcMin));
        porcCompMinTextView.setText(String.format(Locale.getDefault(),"< %.2f %s",porcbasecompMin,simulacaoViewModel.getSimulacao().getValue().getUnidades().get(7)));
        csfMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
        cstMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
        csdaMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
        csdcMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
        porcCompMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
        simulacaoViewModel.getSimulacao().getValue().simular();
        listeners();
    }

    private void listeners() {
        tensaoMaxEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_aviso_unidade),
                            v.getContext().getResources().getString(R.string.simular_tensoes_max_hint), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(6));
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

        tensaoMinEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_aviso_unidade),
                            v.getContext().getResources().getString(R.string.simular_tensoes_min_hint), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(6));
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

        porcCompEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                try{
                    if(format.parse(porcCompEditText.getText().toString()).doubleValue() < porcbasecompMin){
                        porcCompMinTextView.setText(String.format(Locale.getDefault(),"< %.2f %s",porcbasecompMin,simulacaoViewModel.getSimulacao().getValue().getUnidades().get(7)));
                        porcCompMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                    }else if(format.parse(porcCompEditText.getText().toString()).doubleValue() == csfMin){
                        porcCompMinTextView.setText(String.format(Locale.getDefault(),"= %.2f %s",porcbasecompMin,simulacaoViewModel.getSimulacao().getValue().getUnidades().get(7)));
                        porcCompMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }else{
                        porcCompMinTextView.setText(String.format(Locale.getDefault(),"> %.2f %s",porcbasecompMin,simulacaoViewModel.getSimulacao().getValue().getUnidades().get(7)));
                        porcCompMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }
                }catch (NumberFormatException e){
                    porcCompMinTextView.setText(String.format(Locale.getDefault(),"< %.2f %s",porcbasecompMin,simulacaoViewModel.getSimulacao().getValue().getUnidades().get(7)));
                    porcCompMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable s) {
            }
        });

        csfEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                try{
                    if(format.parse(csfEditText.getText().toString()).doubleValue() < csfMin){
                        csfMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csfMin));
                        csfMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                    }else if(format.parse(csfEditText.getText().toString()).doubleValue() == csfMin){
                        csfMinTextView.setText(String.format(Locale.getDefault(),"= %.2f",csfMin));
                        csfMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }else{
                        csfMinTextView.setText(String.format(Locale.getDefault(),"> %.2f",csfMin));
                        csfMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }
                }catch (NumberFormatException e){
                    csfMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csfMin));
                    csfMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable s) {
            }
        });

        cstEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                try{
                    if(format.parse(cstEditText.getText().toString()).doubleValue() < cstMin){
                        cstMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",cstMin));
                        cstMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                    }else if(format.parse(cstEditText.getText().toString()).doubleValue() == cstMin){
                        cstMinTextView.setText(String.format(Locale.getDefault(),"= %.2f",cstMin));
                        cstMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }else{
                        cstMinTextView.setText(String.format(Locale.getDefault(),"> %.2f",cstMin));
                        cstMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }
                }catch (NumberFormatException e){
                    cstMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",cstMin));
                    cstMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable s) {
            }
        });

        csdaEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                try{
                    if(format.parse(csdaEditText.getText().toString()).doubleValue() < csdaMin){
                        csdaMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csdaMin));
                        csdaMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                    }else if(format.parse(csdaEditText.getText().toString()).doubleValue() == csdaMin){
                        csdaMinTextView.setText(String.format(Locale.getDefault(),"= %.2f",csdaMin));
                        csdaMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }else{
                        csdaMinTextView.setText(String.format(Locale.getDefault(),"> %.2f",csdaMin));
                        csdaMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }
                }catch (NumberFormatException e){
                    csdaMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csdaMin));
                    csdaMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable s) {
            }
        });

        csdcEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                try{
                    if(format.parse(csdcEditText.getText().toString()).doubleValue() < csdcMin){
                        csdcMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csdcMin));
                        csdcMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                    }else if(format.parse(csdcEditText.getText().toString()).doubleValue() == csdcMin){
                        csdcMinTextView.setText(String.format(Locale.getDefault(),"= %.2f",csdcMin));
                        csdcMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }else{
                        csdcMinTextView.setText(String.format(Locale.getDefault(),"> %.2f",csdcMin));
                        csdcMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }
                }catch (NumberFormatException e){
                    csdcMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",csdcMin));
                    csdcMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable s) {
            }
        });

        simularButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(fissura){
                    try{
                        csfU = format.parse(csfEditText.getText().toString());
                        cstU = format.parse(cstEditText.getText().toString());
                        csdaU = format.parse(csdaEditText.getText().toString());
                        csdcU = format.parse(csdcEditText.getText().toString());
                        tensaominU = format.parse(tensaoMinEditText.getText().toString());
                        porcbasecompU = format.parse(porcCompEditText.getText().toString());
                        simulacaoViewModel.getSimulacao().getValue().obterFeedbackFissura(csfU.doubleValue(), cstU.doubleValue(), csdaU.doubleValue(), csdcU.doubleValue(), tensaominU.doubleValue(), porcbasecompU.doubleValue());
                        simulacaoViewModel.setSimulacao(simulacaoViewModel.getSimulacao().getValue());
                        feedback="";
                    }catch (Exception e){
                        feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.aviso_campos_nao_preenchidos));
                    }
                }else{
                    try{
                        tensaominU = format.parse(tensaoMinEditText.getText().toString());
                        tensaomaxU = format.parse(tensaoMaxEditText.getText().toString());
                        porcbasecompU = format.parse(porcCompEditText.getText().toString());
                        fissura = simulacaoViewModel.getSimulacao().getValue().obterFeedbackTensaoBase(tensaomaxU.doubleValue(), tensaominU.doubleValue(), porcbasecompU.doubleValue());
                        simulacaoViewModel.setSimulacao(simulacaoViewModel.getSimulacao().getValue());
                        if(fissura){
                            simulacaoViewModel.getSimulacao().getValue().simularFissura();
                            fissuraEditText.setText(String.format(Locale.getDefault(),getResources().getString(R.string.simular_parametros_set_text), getResources().getString(R.string.simular_fissura_hint),
                                    simulacaoViewModel.getSimulacao().getValue().getFissura(), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(0)));
                            tensaoMaxEditText.setEnabled(false);
                            tensaoMaxEditText.setText(String.format(Locale.getDefault(),"%.2f",simulacaoViewModel.getSimulacao().getValue().getTensaomax()));
                            csfEditText.setEnabled(true);
                            cstEditText.setEnabled(true);
                            csdaEditText.setEnabled(true);
                            csdcEditText.setEnabled(true);
                            avisoFissuraTextView.setText(v.getContext().getResources().getString(R.string.simular_com_fissura_aviso));
                            if(simulacaoViewModel.getSimulacao().getValue() instanceof Secao1){
                                Glide.with(v.getContext()).load(ResourcesCompat.getDrawable(getResources(),R.drawable.secao1_fissura,null)).into(fissuraImageView);
                            }else if(simulacaoViewModel.getSimulacao().getValue() instanceof Secao2){
                                Glide.with(v.getContext()).load(ResourcesCompat.getDrawable(getResources(),R.drawable.secao2_fissura,null)).into(fissuraImageView);
                            }else if(simulacaoViewModel.getSimulacao().getValue() instanceof Secao3){
                                Glide.with(v.getContext()).load(ResourcesCompat.getDrawable(getResources(),R.drawable.secao3_fissura,null)).into(fissuraImageView);
                            }else if(simulacaoViewModel.getSimulacao().getValue() instanceof Secao4){
                                Glide.with(v.getContext()).load(ResourcesCompat.getDrawable(getResources(),R.drawable.secao4_fissura,null)).into(fissuraImageView);
                            }
                        }
                        feedback="";
                    }catch (Exception e){
                        feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.aviso_campos_nao_preenchidos));
                    }
                }
                if(!feedback.isEmpty()){
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

    }
}