package com.example.dams;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link TombamentoFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class TombamentoFragment extends Fragment {

    private Button simularButton;
    private String feedback;
    private EditText somamestEditText, somamdesEditText, cstEditText;
    private TextView cstMinTextView;
    private SimulacaoViewModel simulacaoViewModel;
    private Number somamestU, somamdesU, cstU;
    private double cstMin;
    private NumberFormat format;

    public TombamentoFragment() {
        // Required empty public constructor
    }

    // TODO: Rename and change types and number of parameters
    public static TombamentoFragment newInstance() {
        return new TombamentoFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        simulacaoViewModel = new ViewModelProvider(getActivity()).get(SimulacaoViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_tombamento, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        simularButton = getView().findViewById(R.id.tombamentoButton);
        somamestEditText = getView().findViewById(R.id.tombamentoSEstEditText);
        somamdesEditText = getView().findViewById(R.id.tombamentoSDesEditText);
        cstEditText = getView().findViewById(R.id.tombamentoCSTEditText);
        cstMinTextView = getView().findViewById(R.id.tombamentoCSTminTextView);

        format = NumberFormat.getInstance(Locale.getDefault());
        try {
            cstMin = format.parse(simulacaoViewModel.getSimulacao().getValue().getCmin().get(1)).doubleValue();
        } catch (ParseException e) {
            e.printStackTrace();
        }

        cstMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",cstMin));
        cstMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
        simulacaoViewModel.getSimulacao().getValue().simular();
        listeners();
    }

    private void listeners() {
        somamestEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_aviso_unidade),
                            v.getContext().getResources().getString(R.string.simular_tombamento_est_hint), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(4));
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

        somamdesEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.selecionar_secao_aviso_unidade),
                            v.getContext().getResources().getString(R.string.simular_tombamento_desest_hint), simulacaoViewModel.getSimulacao().getValue().getUnidades().get(4));
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

        cstEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                try{
                    if(format.parse(cstEditText.getText().toString()).doubleValue() < cstMin){
                        cstMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",cstMin));
                        cstMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                    }else if(format.parse(cstEditText.getText().toString()).doubleValue() == cstMin){
                        cstMinTextView.setText(String.format(Locale.getDefault(),"= %.2f",cstMin));
                        cstMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }else{
                        cstMinTextView.setText(String.format(Locale.getDefault(),"> %.2f",cstMin));
                        cstMinTextView.setTextColor(getResources().getColor(R.color.lime));
                    }
                }catch (NumberFormatException e){
                    cstMinTextView.setText(String.format(Locale.getDefault(),"< %.2f",cstMin));
                    cstMinTextView.setTextColor(getResources().getColor(R.color.design_default_color_error));
                }catch (Exception e){
                }
            }
            @Override
            public void afterTextChanged(Editable s) {
            }
        });

        simularButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try{
                    somamestU = format.parse(somamestEditText.getText().toString());
                    somamdesU = format.parse(somamdesEditText.getText().toString());
                    cstU = format.parse(cstEditText.getText().toString());
                    simulacaoViewModel.getSimulacao().getValue().obterFeedbackTombamento(somamestU.doubleValue(), somamdesU.doubleValue(), cstU.doubleValue());
                    simulacaoViewModel.setSimulacao(simulacaoViewModel.getSimulacao().getValue());
                    feedback="";
                }catch (Exception e){
                    feedback = String.format(Locale.getDefault(),v.getContext().getResources().getString(R.string.aviso_campos_nao_preenchidos));
                }

                if(!feedback.isEmpty()){
                    Toast.makeText(v.getContext(), feedback, Toast.LENGTH_LONG).show();
                }
            }
        });

    }
}